# Compressed Databases
Authors: **Ahmed M Moustafa and Paul J Planet**

## Rules

Each Compressed Database Release has more genomes analyzed from GISAID, new STs and CCs may be assigned and older CCs may be renamed.


GISAID Sequences inclusion criteria in this analysis.
  - Complete, high coverage genomes (excluding low coverage ) were downloaded from GISAID.
  - Genomes had at least 29,000 bp in length and had less than 1% Ns.
  - Any genome that had any ambiguity (any letter other than A,T,G and C) in the 10 open reading frames (ORF) was excluded.

The STs identified by GNUVID were grouped into larger taxonomic units, clonal complexes (CCs), defined as clusters of >20 STs that were single or double allele variants (SLV or DLV) away from a “founder” as defined in a a minimum spanning tree (MST) using the goeBURST algorithm.

## Limitations

- Because a change in any allele creates a new ST our method may accumulate and count “unnecessary” STs that have been seen only once or may be due to a sequencing error. This is partially ameliorated by the use of the CC definition that allows some variability amongst the members of a group. A large number of STs also may allow more granular approaches to tracking new lineages.

- The status of the CCs (active, silent and inactive) may be a sampling issue.

- Certain regions (US and Europe) clearly sequenced more genomes compared to other regions.

**The table below shows summary information of the 564 Active Clonal Complexes (CCs).**

The "Most common 5 countries" and "Most common Region" columns show the five countries and the region, respectively, with the most sequences from each Clonal Complex. Starting from the DB update (06/21/2021), the GISAID clades are reported for each CC for easier correlation between the two systems. **A full report for the 4084 CCs can be found [here](https://github.com/ahmedmagds/GNUVID/blob/master/db/GNUVID_08312021_CCs_report.txt)**.


| Clonal Complex | Number of STs/isolates | Most common 5 countries | Most common Region | Date range | Status | Pango Lineage (%isolates) | GISAID Clade (%) | WHO Naming | AA Substitutions in >=60% of isolates |
|----------------|-------------------------|-------------------------|--------------------|------------|--------|---------------------------|------------------|------------|---------------------------------------|
| **255** | 4915/10690 | Turkey (25%), USA (17%), United Kingdom (8%), India (4%), Spain (4%) | Europe (66%) | 2020-01-24 to 2021-08-20 | **Active** | B.1 (52%) | G (97%) | NA | Spike_D614G (99%),NSP12_P323L (94%) |
| **45062** | 35193/77209 | United Kingdom (45%), Japan (12%), Germany (5%), USA (5%), Sweden (4%) | Europe (81%) | 2020-09-20 to 2021-08-18 | **Active** | B.1.1.7 (99%) | GRY (95%) | Alpha | Spike_D614G (100%),Spike_N501Y (100%),Spike_S982A (100%),Spike_A570D (100%),N_S235F (100%),NSP12_P323L (100%),NSP3_A890D (100%),NSP3_T183I (100%),Spike_T716I (100%),Spike_P681H (100%),Spike_D1118H (100%),Spike_V70del (100%),NS8_Q27stop (100%),N_R203K (100%),N_D3L (100%),NS8_R52I (100%),Spike_H69del (100%),NS8_Y73C (100%),N_G204R (100%),Spike_Y144del (100%),NSP3_I1412T (100%),NSP6_G107del (99%),NSP6_F108del (99%),NSP6_S106del (99%) |
| **58534** | 16792/33585 | USA (26%), United Kingdom (23%), Canada (12%), Sweden (8%), Germany (5%) | Europe (61%) | 2020-03-12 to 2021-08-20 | **Active** | B.1.1.7 (100%) | GRY (92%) | Alpha | Spike_D614G (100%),NS8_Y73C (100%),NS8_R52I (100%),Spike_T716I (100%),Spike_N501Y (100%),NS8_Q27stop (100%),Spike_S982A (100%),NSP3_A890D (100%),Spike_A570D (100%),NSP3_T183I (100%),N_S235F (100%),Spike_P681H (100%),NS8_K68stop (100%),Spike_H69del (100%),Spike_D1118H (100%),Spike_V70del (100%),N_D3L (100%),NSP3_I1412T (100%),NSP12_P323L (100%),N_R203K (100%),Spike_Y144del (100%),NSP6_G107del (99%),NSP6_S106del (99%),NSP6_F108del (99%),N_G204R (98%) |
| **75478** | 840/1474 | France (34%), United Kingdom (28%), Italy (6%), Switzerland (5%), USA (5%) | Europe (91%) | 2020-12-05 to 2021-08-18 | **Active** | B.1.1.7 (99%) | GRY (96%) | Alpha | Spike_H69del (100%),Spike_T716I (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),NSP3_A890D (100%),Spike_D1118H (100%),NS8_Y73C (100%),Spike_V70del (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NS8_Q27stop (100%),NSP3_T183I (100%),N_R203K (100%),NSP12_P323L (100%),NSP6_S106del (100%),NSP6_G107del (100%),N_G204R (100%),Spike_Y144del (100%),NSP6_F108del (100%),Spike_S98F (100%) |
| **81085** | 7876/10479 | Brazil (62%), USA (18%), Chile (5%), Canada (3%), Italy (2%) | South America (70%) | 2020-10-01 to 2021-08-19 | **Active** | P.1 (94%) | GR (99%) | Gamma | Spike_D614G (100%),NSP3_K977Q (100%),Spike_N501Y (100%),N_R203K (100%),NS8_E92K (100%),Spike_E484K (100%),Spike_H655Y (100%),Spike_T20N (100%),N_P80R (100%),Spike_L18F (100%),N_G204R (100%),Spike_P26S (100%),Spike_R190S (100%),NSP3_S370L (100%),NSP12_P323L (100%),Spike_V1176F (100%),NSP13_E341D (100%),Spike_T1027I (100%),NS3_S253P (99%),NSP6_S106del (99%),NSP6_G107del (99%),NSP6_F108del (99%),Spike_D138Y (98%),Spike_K417T (98%) |
| **115066** | 907/1096 | Indonesia (86%), Malaysia (5%), Singapore (3%), Japan (2%), Australia (1%) | Asia (97%) | 2020-05-26 to 2021-08-17 | **Active** | B.1.466.2 (93%) | GH (100%) | VUM | NSP12_P323L (100%),Spike_D614G (100%),N_T205I (100%),NS3_Q57H (100%),NSP3_T350I (99%),NSP3_P822L (99%),Spike_N439K (98%),NSP3_S126L (98%),NSP13_S259L (98%),Spike_P681R (97%),NSP6_L75F (91%) |
| **142037** | 577/940 | Germany (29%), Netherlands (23%), Israel (19%), Poland (18%), Denmark (6%) | Europe (80%) | 2020-12-20 to 2021-08-17 | **Active** | B.1.1.7 (99%) | GRY (96%) | Alpha | Spike_T716I (100%),Spike_F490S (100%),Spike_N501Y (100%),Spike_D1118H (100%),NS3_T151I (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),Spike_H69del (100%),NSP6_S106del (100%),NSP3_I1412T (100%),Spike_P681H (100%),NSP6_G107del (100%),NSP6_F108del (100%),N_S235F (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_A570D (100%),NS8_R52I (100%),Spike_Y144del (100%),NS8_Y73C (100%),N_R203K (100%),NSP3_A890D (100%),Spike_V70del (100%),Spike_L5F (100%),N_G204R (100%),NS7a_Q62stop (99%) |
| **233088** | 89/127 | Turkey (28%), USA (25%), India (21%), Serbia (7%), Austria (4%) | Europe (48%) | 2020-03-14 to 2021-08-21 | **Active** | B.1 (35%) | G (90%) | NA | N_R203M (100%),NSP12_P323L (100%),Spike_D614G (100%) |
| **268709** | 58/88 | Malaysia (77%), Thailand (14%), Singapore (3%), Germany (3%), Slovakia (1%) | Asia (94%) | 2021-02-01 to 2021-08-17 | **Active** | B.1.351 (100%) | GH (100%) | Beta | E_P71L (100%),NSP3_K837N (100%),Spike_K417N (100%),NSP6_S106del (100%),Spike_E484K (100%),Spike_A701V (100%),Spike_N501Y (100%),M_H155Y (100%),Spike_A243del (100%),Spike_L18F (100%),Spike_D215G (100%),Spike_L244del (100%),NSP6_G107del (100%),N_T205I (100%),NSP6_F108del (100%),Spike_L242del (100%),NS3_Q57H (100%),NSP6_M164V (100%),NSP2_T85I (100%),Spike_D80A (100%),NSP12_P323L (100%),N_Q9H (100%),NSP5_K90R (100%),Spike_D614G (100%),NS3_S171L (100%),NSP5_K100R (94%),NS3_A72S (94%) |
| **324988** | 55/65 | Peru (35%), USA (34%), Ecuador (17%), Chile (14%) | South America (66%) | 2021-02-08 to 2021-08-17 | **Active** | C.37 (100%) | GR (100%) | Lambda | Spike_L249del (100%),Spike_S247del (100%),Spike_G75V (100%),Spike_Y248del (100%),NSP6_S106del (100%),N_R203K (100%),Spike_F490S (100%),Spike_T859N (100%),N_P13L (100%),Spike_D253N (100%),Spike_R246del (100%),NSP4_L438P (100%),NSP4_T492I (100%),N_G214C (100%),NSP5_G15S (100%),NSP6_G107del (100%),NSP3_F1569V (100%),NSP3_T428I (100%),NSP6_F108del (100%),N_G204R (100%),NSP12_P323L (100%),Spike_G252del (100%),Spike_D614G (100%),Spike_T250del (100%),Spike_L452Q (100%),Spike_P251del (100%),Spike_T76I (100%),NSP3_P1469S (98%) |
| **341906** | 581/1581 | Japan (99%), Italy (0%), United Kingdom (0%), USA (0%), Canada (0%) | Asia (99%) | 2021-02-16 to 2021-08-24 | **Active** | B.1.1.7 (100%) | GRY (64%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NS8_Y73C (100%),Spike_V70del (100%),NSP12_P323L (100%),NS8_Q72H (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NSP6_S106del (100%),NSP6_G107del (100%),NSP6_F108del (100%),N_G204R (100%),Spike_T33I (99%) |
| **343219** | 159/304 | France (48%), Sweden (30%), Germany (17%), Belgium (2%), USA (1%) | Europe (99%) | 2021-02-06 to 2021-08-20 | **Active** | B.1.1.7 (100%) | GRY (81%) | Alpha | NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP3_A890D (100%),Spike_D1118H (100%),N_G204R (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NS8_K68stop (100%),Spike_E484K (100%),NS8_Y73C (100%),Spike_H69del (99%),Spike_V70del (99%),NSP6_S106del (98%),NSP6_G107del (98%),NSP6_F108del (98%),NSP1_K58R (68%) |
| **363620** | 67/75 | India (15%), South Korea (12%), USA (12%), United Kingdom (11%), Japan (8%) | Asia (48%) | 2021-01-07 to 2021-08-25 | **Active** | B.1.617.2 (97%) | GK (97%) | Delta | NS7a_L116F (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),NS3_Q57R (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),N_D63G (97%),Spike_D950N (97%),Spike_G142D (72%) |
| **380622** | 174/544 | United Kingdom (77%), Canada (7%), India (3%), USA (3%), Sweden (2%) | Europe (84%) | 2021-03-29 to 2021-08-21 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_D950N (100%),NS7a_V82A (99%),Spike_A222V (99%),Spike_G142D (94%) |
| **386479** | 274/502 | Thailand (86%), Cambodia (5%), Israel (3%), Japan (1%), Finland (1%) | Asia (96%) | 2021-03-01 to 2021-08-22 | **Active** | B.1.1.7 (100%) | GRY (95%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NS8_Y73C (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_S982A (100%),NSP6_S106del (100%),N_R203K (100%),NS8_R52I (100%),NSP6_G107del (100%),NSP6_F108del (100%),Spike_D614G (100%),N_D3L (100%),N_S235F (100%),NS8_K68stop (98%),NS3_G254stop (96%) |
| **387085** | 855/1321 | USA (20%), United Kingdom (14%), India (14%), Italy (8%), Singapore (6%) | Europe (39%) | 2021-01-08 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | N_R203M (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_A222V (100%),Spike_E156G (100%),Spike_T478K (100%),Spike_D614G (100%),Spike_L452R (100%),NSP12_G671S (100%),NS3_S26L (100%),NS7a_V82A (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_T19R (100%),NSP3_P822L (100%),NSP4_A446V (100%),N_D63G (100%),NSP6_T181I (100%),Spike_D950N (100%),NSP13_P77L (100%),NSP12_P323L (100%),Spike_G142D (98%) |
| **395705** | 629/942 | United Kingdom (13%), India (13%), USA (12%), Turkey (10%), Germany (8%) | Europe (52%) | 2021-03-04 to 2021-08-18 | **Active** | B.1.617.2 (94%) | GK (96%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_T19R (100%),N_D377Y (100%),NSP13_P77L (100%),NSP4_A446V (100%),NSP12_P323L (100%),Spike_D614G (100%),NS7a_L116F (100%),NS7a_V82A (100%),NS7a_T120I (100%),M_I82T (100%),Spike_L452R (100%),N_D63G (100%),Spike_T478K (100%),N_R385K (100%),NSP6_V149A (100%),NSP15_H234Y (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),NSP2_P129L (99%),NSP3_H1274Y (98%),NSP3_P822L (98%),Spike_D950N (98%),Spike_G142D (78%) |
| **397979** | 152/186 | Cambodia (97%), South Korea (3%) | Asia (100%) | 2021-02-19 to 2021-08-17 | **Active** | B.1.1.7 (99%) | GRY (95%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),NSP6_S106del (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP6_G107del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NSP6_F108del (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NS8_Y73C (95%) |
| **404772** | 31/115 | USA (97%), Portugal (3%), Brazil (1%) | North America (97%) | 2021-04-11 to 2021-08-17 | **Active** | B.1.617.2 (68%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),NSP12_F192V (99%),NSP3_A1736V (99%) |
| **406497** | 79/111 | Mexico (49%), USA (17%), Spain (12%), United Kingdom (5%), France (4%) | North America (66%) | 2021-04-12 to 2021-08-17 | **Active** | AY.12 (93%) | GK (93%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%),NSP3_P1469S (99%),Spike_T95I (98%),NSP2_A419V (62%) |
| **414828** | 294/1035 | United Kingdom (100%), Japan (0%), Netherlands (0%), USA (0%) | Europe (100%) | 2021-04-14 to 2021-08-20 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1469S (100%) |
| **417415** | 89/104 | Brazil (100%) | South America (100%) | 2021-04-14 to 2021-08-18 | **Active** | P.1.7 (99%) | GR (100%) | Gamma | Spike_R190S (100%),NSP6_S106del (100%),Spike_H655Y (100%),Spike_E484K (100%),N_R203K (100%),Spike_T1027I (100%),Spike_N501Y (100%),Spike_P681H (100%),NS3_S253P (100%),Spike_L18F (100%),N_P80R (100%),Spike_T20N (100%),NSP3_K977Q (100%),NSP6_G107del (100%),NSP6_F108del (100%),NSP3_S370L (100%),N_G204R (100%),Spike_P26S (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1176F (100%),NSP13_E341D (100%),Spike_K417T (99%),Spike_D138Y (99%),NS8_E92K (99%) |
| **421698** | 260/323 | India (18%), USA (15%), Germany (14%), Italy (13%), Vietnam (6%) | Europe (53%) | 2021-03-29 to 2021-08-24 | **Active** | B.1.617.2 (80%) | GK (96%) | Delta | N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),Spike_P681R (100%),Spike_T19R (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),NSP4_A446V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP3_P822L (100%),Spike_T478K (100%),Spike_L452R (100%),NS3_S26L (99%),NS7a_V82A (99%),NS7a_T120I (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_D950N (99%),Spike_E156G (99%),N_D63G (98%) |
| **423255** | 195/582 | Turkey (33%), United Kingdom (31%), Belgium (9%), USA (4%), Portugal (4%) | Europe (87%) | 2021-04-03 to 2021-08-22 | **Active** | B.1.617.2 (85%) | GK (99%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_T19R (100%),M_I82T (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_G215C (100%),N_R203M (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),Spike_D950N (100%),NSP6_T77A (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_G142D (89%),NS7b_T40I (77%),Spike_T95I (69%) |
| **423259** | 41/178 | United Kingdom (62%), USA (31%), Russia (3%), Denmark (2%), Japan (1%) | Europe (69%) | 2021-04-16 to 2021-08-21 | **Active** | B.1.617.2 (79%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_P45L (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%),NSP3_A579V (99%),Spike_G142D (78%) |
| **425463** | 441/769 | Denmark (28%), Russia (14%), United Kingdom (14%), Turkey (9%), USA (7%) | Europe (89%) | 2021-04-17 to 2021-08-21 | **Active** | B.1.617.2 (94%) | GK (98%) | Delta | N_G215C (100%),NSP12_G671S (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP3_P1469S (100%),NS7a_P45L (100%),NSP12_P323L (100%),N_R203M (100%),Spike_R158del (100%),Spike_F157del (100%),NSP14_A394V (100%),NSP2_K81N (100%),Spike_E156G (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),NS3_S26L (100%),N_D377Y (100%),NS7a_V82A (100%),Spike_T19R (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP6_T77A (99%),Spike_D950N (98%),Spike_G142D (95%) |
| **427161** | 1207/5226 | United Kingdom (98%), USA (0%), Italy (0%), Turkey (0%), Sweden (0%) | Europe (99%) | 2021-04-18 to 2021-08-22 | **Active** | AY.4 (61%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NS7a_V82A (100%),Spike_T19R (100%),NSP13_P77L (100%),Spike_L452R (100%),Spike_P681R (100%),Spike_D950N (100%),NSP12_P323L (100%),N_D63G (100%),Spike_T95I (100%),NSP3_P1228L (100%),NSP4_T492I (100%),NSP4_V167L (100%),Spike_G142D (100%),NSP3_A1711V (99%) |
| **427290** | 23862/48597 | United Kingdom (88%), USA (4%), Turkey (3%), Denmark (1%), Germany (0%) | Europe (95%) | 2020-11-04 to 2021-08-24 | **Active** | B.1.617.2 (76%) | GK (99%) | Delta | NS3_S26L (100%),M_I82T (100%),Spike_T478K (100%),N_R203M (100%),Spike_D614G (100%),N_D63G (100%),N_D377Y (100%),Spike_L452R (100%),N_G215C (100%),NSP12_G671S (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_T19R (100%),NSP12_P323L (100%),Spike_P681R (100%),NSP4_T492I (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_D950N (100%),NSP13_P77L (100%),NSP14_A394V (100%),NSP3_P1228L (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NS7b_T40I (100%),NSP3_A488S (99%),NSP4_V167L (99%),Spike_G142D (98%),Spike_T95I (97%),NSP3_A1711V (89%) |
| **429199** | 187/1055 | United Kingdom (95%), Italy (2%), USA (2%), Sweden (0%), Belgium (0%) | Europe (98%) | 2021-04-19 to 2021-08-23 | **Active** | B.1.617.2 (79%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_D511N (94%),NSP13_E365D (94%) |
| **433414** | 2041/4091 | United Kingdom (79%), USA (5%), Portugal (3%), France (2%), Denmark (2%) | Europe (94%) | 2021-02-13 to 2021-08-23 | **Active** | B.1.617.2 (60%) | GK (99%) | Delta | NS3_S26L (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_R203M (100%),NSP12_G671S (100%),NSP4_T492I (100%),Spike_T19R (100%),NSP4_V167L (100%),NSP6_T77A (100%),N_D63G (100%),Spike_P681R (100%),Spike_L452R (100%),N_G215C (100%),Spike_R158del (100%),Spike_E156G (100%),Spike_F157del (100%),NS7a_V82A (100%),Spike_D950N (99%),N_S327L (96%),Spike_G142D (94%) |
| **440067** | 103/148 | USA (94%), Mexico (3%), Italy (2%), Japan (1%) | North America (97%) | 2021-04-21 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NS8_S67F (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_D377Y (99%),Spike_D950N (99%),NSP2_G339S (99%),NSP6_T181I (99%),Spike_G142D (93%) |
| **442571** | 151/191 | USA (49%), Mexico (49%), Israel (1%), Spain (1%), South Korea (1%) | North America (98%) | 2021-04-11 to 2021-08-18 | **Active** | B.1.617.2 (94%) | GK (95%) | Delta | Spike_V1264L (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS6_K48N (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_R158del (99%),NS7a_V82A (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_D950N (98%) |
| **446150** | 990/1665 | USA (99%), Germany (0%), Mexico (0%), Canada (0%), Curacao (0%) | North America (100%) | 2021-04-23 to 2021-08-17 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),N_D377Y (100%),Spike_S112L (100%),Spike_E156G (100%),NSP12_P323L (100%),Spike_L452R (100%),NS3_S26L (100%),NS7a_T120I (100%),NSP6_T77A (100%),NS3_E239Q (100%),Spike_D950N (100%),Spike_G142D (95%) |
| **449110** | 215/693 | United Kingdom (100%), Portugal (0%) | Europe (100%) | 2021-04-25 to 2021-08-17 | **Active** | B.1.617.2 (89%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),Spike_G1124V (100%),Spike_L216F (100%) |
| **450463** | 21/32 | United Kingdom (84%), Germany (12%), Kuwait (3%) | Europe (97%) | 2021-04-26 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (84%) |
| **453829** | 454/1003 | USA (92%), United Kingdom (4%), Turkey (1%), Germany (0%), Netherlands (0%) | North America (92%) | 2021-04-18 to 2021-08-22 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP4_T492I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),Spike_L452R (100%),N_D63G (100%),Spike_T19R (100%),NSP12_P323L (100%),NS7a_V82A (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),NS7a_T120I (100%),NSP6_T77A (100%),NSP3_P1228L (99%),NS7b_T40I (99%),NSP14_A394V (99%),Spike_D950N (99%),NSP3_A488S (99%),NSP4_V167L (99%),Spike_G142D (95%),NSP12_F192V (92%),NSP3_A1736V (90%) |
| **458158** | 1427/2729 | Turkey (21%), Denmark (11%), Russia (10%), Sweden (10%), Netherlands (9%) | Europe (87%) | 2021-02-07 to 2021-08-24 | **Active** | B.1.617.2 (91%) | GK (98%) | Delta | M_I82T (100%),N_R203M (100%),NSP3_P1228L (100%),NSP4_T492I (100%),N_D377Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_D614G (100%),N_D63G (100%),NSP3_P1469S (100%),N_G215C (100%),NS3_S26L (100%),Spike_P681R (100%),NSP12_G671S (100%),NSP3_A488S (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_V82A (100%),Spike_T478K (100%),Spike_F157del (100%),Spike_R158del (100%),Spike_E156G (100%),NSP6_T77A (99%),NS7a_T120I (99%),NSP12_P323L (99%),NS7a_P45L (99%),NSP2_K81N (99%),Spike_L452R (98%),Spike_D950N (98%),NSP4_V167L (97%),Spike_G142D (93%) |
| **458750** | 1115/2462 | United Kingdom (94%), USA (4%), Denmark (0%), South Africa (0%), Canada (0%) | Europe (95%) | 2021-04-18 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_D614G (100%),NSP3_A488S (100%),NSP14_A394V (100%),N_D377Y (100%),NS7b_T40I (100%),Spike_T478K (100%),NSP12_P323L (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),Spike_T95I (100%),NSP3_P1469S (100%),NSP3_S1230F (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_D950N (100%),NSP6_T77A (99%),Spike_G142D (99%) |
| **460247** | 119/208 | USA (85%), Turkey (10%), India (1%), Germany (1%), Australia (1%) | North America (85%) | 2021-04-03 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | NSP4_D217N (100%),N_D63G (100%),N_R203M (100%),NS3_S26L (100%),Spike_K77T (100%),NSP4_F375S (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP3_P822L (100%),Spike_E156G (100%),NSP15_K259R (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP2_P129L (100%),Spike_P681R (100%),Spike_D950N (100%),NSP6_H11Q (82%),NSP3_P1103Q (81%) |
| **463902** | 89/169 | USA (92%), United Kingdom (7%), Sweden (1%) | North America (92%) | 2021-04-29 to 2021-08-18 | **Active** | B.1.617.2 (76%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),Spike_D950N (98%),NSP12_F192V (93%),Spike_G142D (83%) |
| **467245** | 302/1095 | United Kingdom (97%), Portugal (1%), Turkey (1%), Netherlands (0%), Denmark (0%) | Europe (100%) | 2021-05-01 to 2021-08-22 | **Active** | AY.4 (77%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (100%),Spike_T95I (100%),NSP4_V167L (99%),Spike_G142D (99%),NSP3_A1711V (99%) |
| **467353** | 121/277 | United Kingdom (99%), France (1%), Japan (0%) | Europe (100%) | 2021-05-01 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **474290** | 51/118 | United Kingdom (99%), Netherlands (1%) | Europe (100%) | 2021-05-04 to 2021-08-22 | **Active** | B.1.617.2 (90%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (99%) |
| **474310** | 455/1014 | United Kingdom (96%), USA (2%), France (1%), Denmark (0%), Netherlands (0%) | Europe (98%) | 2021-05-03 to 2021-08-22 | **Active** | B.1.617.2 (83%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_G142D (100%),NSP12_G671S (99%),Spike_T95I (97%),NSP3_A1711V (97%) |
| **479388** | 74/129 | USA (36%), Russia (25%), Germany (10%), Finland (5%), France (4%) | Europe (63%) | 2021-04-22 to 2021-08-19 | **Active** | B.1.617.2 (78%) | GK (95%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (99%),Spike_D950N (98%),NS7a_P45L (98%),N_D63G (95%),Spike_G142D (78%) |
| **480083** | 52/134 | United Kingdom (100%) | Europe (100%) | 2021-04-27 to 2021-08-21 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP12_I244V (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%) |
| **480144** | 102/311 | United Kingdom (99%), USA (1%), Netherlands (0%) | Europe (99%) | 2021-05-06 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (98%) |
| **481750** | 100/390 | Singapore (72%), United Kingdom (27%), France (1%) | Asia (72%) | 2021-05-06 to 2021-08-20 | **Active** | B.1.617.2 (99%) | GK (95%) | Delta | Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),NSP2_T223I (100%),N_G214del (100%),NS3_S26L (100%),NSP5_V296I (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),N_G215del (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (100%),Spike_D950N (100%),Spike_G142D (99%) |
| **482497** | 210/520 | United Kingdom (100%), France (0%) | Europe (100%) | 2021-05-07 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T51I (100%),Spike_G142D (100%),NSP12_P323L (100%) |
| **483312** | 30/108 | United Kingdom (95%), Sweden (1%), Norway (1%), Netherlands (1%), France (1%) | Europe (100%) | 2021-05-07 to 2021-08-18 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **483317** | 65/125 | United Kingdom (96%), USA (4%) | Europe (96%) | 2021-05-07 to 2021-08-21 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),N_G215C (96%),NSP3_A1711V (96%),NS7b_T40I (96%) |
| **483457** | 130/191 | USA (99%), Sweden (1%) | North America (99%) | 2021-04-30 to 2021-08-17 | **Active** | AY.4 (96%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_S112L (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NSP13_P77L (99%),NS3_E239Q (99%),Spike_D950N (99%) |
| **484348** | 432/572 | USA (39%), Mexico (15%), Spain (9%), Ecuador (7%), Aruba (7%) | North America (57%) | 2021-01-21 to 2021-08-19 | **Active** | B.1.621 (100%) | GH (99%) | Mu | NSP13_P419S (100%),Spike_T95I (100%),Spike_R346K (100%),N_T205I (100%),NSP3_T237A (100%),Spike_D614G (100%),NS8_P38S (100%),NSP4_T492I (100%),NS8_T11K (100%),Spike_P681H (100%),NSP6_Q160R (100%),NSP3_T720I (100%),Spike_N501Y (100%),NSP12_P323L (100%),NS3_Q57H (100%),Spike_E484K (99%),NS3_V256I (99%),Spike_D950N (95%),NS8_S67F (92%),Spike_ins145N (92%),Spike_Y145S (92%),Spike_Y144T (92%) |
| **485173** | 50/142 | United Kingdom (100%) | Europe (100%) | 2021-05-08 to 2021-08-22 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NS3_R68I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),NSP3_A1711V (99%),NS7a_V82A (96%) |
| **485193** | 25/34 | USA (47%), United Kingdom (41%), Japan (6%), Mexico (3%), Indonesia (3%) | North America (50%) | 2021-05-08 to 2021-08-21 | **Active** | B.1.617.2 (97%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS6_P57L (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (91%) |
| **486510** | 85/286 | United Kingdom (99%), Denmark (1%), Germany (0%) | Europe (100%) | 2021-05-09 to 2021-08-21 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | NS7a_A79D (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP9_Q11H (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7a_C58F (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%) |
| **487649** | 48/71 | United Kingdom (99%), Switzerland (1%) | Europe (100%) | 2021-05-10 to 2021-08-22 | **Active** | B.1.617.2 (65%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_P26S (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_S100C (73%),NSP3_L1511V (62%) |
| **488920** | 79/139 | United Kingdom (71%), Sint Maarten (16%), USA (5%), Netherlands (3%), Greece (1%) | Europe (78%) | 2020-12-20 to 2021-08-18 | **Active** | B.1.1.7 (100%) | GRY (90%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),NSP6_S106del (100%),N_G243C (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),NSP6_G107del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NSP6_F108del (100%),NS8_Y73C (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),Spike_Y144del (99%),NSP2_L550F (71%),NSP6_M192V (71%) |
| **490615** | 56/64 | Sri Lanka (78%), Sweden (8%), Japan (5%), USA (5%), United Kingdom (2%) | Asia (84%) | 2021-04-06 to 2021-08-20 | **Active** | B.1.1.7 (95%) | GRY (73%) | Alpha | Spike_H69del (100%),NSP3_T183I (100%),NS8_K68stop (100%),NSP3_I1412T (100%),Spike_P681H (100%),NSP3_A890D (100%),NS8_Y73C (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP8_T145I (98%),Spike_T716I (98%),NSP6_S106del (98%),Spike_A570D (98%),Spike_N501Y (98%),NS8_R52I (98%),NSP6_G107del (98%),Spike_D1118H (98%),NSP6_F108del (98%),NS3_E194D (98%),Spike_S982A (98%),NS8_Q27stop (97%),Spike_Y144del (97%),N_R203K (95%),N_D3L (94%),N_G204R (92%),N_S235F (91%) |
| **491056** | 105/233 | United Kingdom (100%) | Europe (100%) | 2021-05-11 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **491312** | 76/293 | United Kingdom (100%), India (0%) | Europe (100%) | 2021-05-11 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (98%) |
| **491475** | 546/899 | Portugal (70%), Spain (8%), France (5%), Germany (3%), Belgium (3%) | Europe (95%) | 2021-05-11 to 2021-08-19 | **Active** | AY.5 (53%) | GK (99%) | Delta | NSP12_P323L (100%),Spike_D614G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_L452R (100%),N_R203M (100%),N_D377Y (100%),NSP4_V167L (100%),Spike_R158del (100%),N_S327L (100%),Spike_F157del (100%),Spike_D950N (100%),Spike_E156G (100%),N_G215C (99%),N_D63G (99%),NS7a_V82A (96%) |
| **492175** | 87/190 | USA (90%), United Kingdom (10%) | North America (90%) | 2021-05-02 to 2021-08-23 | **Active** | B.1.617.2 (79%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T19R (99%),Spike_D950N (96%),NSP14_A323S (87%),NSP4_A92V (87%),Spike_G142D (81%) |
| **493591** | 238/383 | Netherlands (53%), United Kingdom (29%), Portugal (7%), Denmark (3%), France (2%) | Europe (99%) | 2021-03-07 to 2021-08-19 | **Active** | B.1.617.2 (78%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_E156G (100%),NSP6_T77A (100%),Spike_D950N (99%),NSP3_P1469S (99%),NSP3_A1711V (99%),NSP13_E319D (96%),NSP12_M463I (96%),Spike_G142D (91%) |
| **493662** | 97/449 | United Kingdom (100%), Germany (0%) | Europe (100%) | 2021-05-10 to 2021-08-21 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (100%),Spike_G142D (100%),NSP3_A1711V (98%),NSP13_M576I (91%) |
| **496199** | 1209/2029 | USA (99%), Switzerland (0%), Germany (0%), United Kingdom (0%), Aruba (0%) | North America (99%) | 2021-05-13 to 2021-08-21 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),NS7b_T40I (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),N_R203M (100%),Spike_P681R (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NSP13_P77L (100%),NSP4_T492I (100%),NSP3_P1469S (100%),N_G215C (100%),NSP6_T77A (100%),NS7a_V82A (100%),NSP13_I334V (100%),M_I82T (99%),Spike_D950N (99%),Spike_G142D (95%) |
| **496273** | 856/1491 | USA (65%), United Kingdom (14%), India (5%), Japan (4%), Turkey (2%) | North America (67%) | 2021-03-05 to 2021-08-18 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | N_R203M (100%),NS7a_V82A (100%),M_I82T (100%),Spike_D614G (100%),NS3_S26L (100%),N_D377Y (100%),NSP12_P323L (100%),Spike_P681R (100%),Spike_T478K (100%),Spike_L452R (100%),NSP4_F375S (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_P129L (99%),NSP3_P822L (99%),N_D63G (99%),NSP15_K259R (99%),NSP4_D217N (99%),Spike_D950N (98%),NSP6_H11Q (98%),Spike_K77T (97%),Spike_R158del (94%),Spike_F157del (94%),Spike_E156G (94%),Spike_G142D (85%),NSP3_P1103Q (69%) |
| **497323** | 29/50 | United Kingdom (84%), USA (16%) | Europe (84%) | 2021-05-14 to 2021-08-22 | **Active** | AY.4 (60%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NS7a_A105V (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_G142D (94%),NSP3_A1711V (84%),Spike_T95I (84%) |
| **497360** | 162/476 | United Kingdom (100%) | Europe (100%) | 2021-05-14 to 2021-08-20 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS7a_I88T (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **497517** | 148/315 | United Kingdom (95%), Denmark (2%), Kosovo (1%), Switzerland (1%), Germany (1%) | Europe (100%) | 2021-05-14 to 2021-08-24 | **Active** | B.1.617.2 (79%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (100%),N_D63G (99%),Spike_G142D (99%) |
| **498845** | 50/103 | United Kingdom (96%), USA (2%), Denmark (2%) | Europe (98%) | 2021-04-27 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7a_V24F (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_G142D (97%),NSP3_A1711V (97%),NSP14_M169T (95%),NSP4_V3F (95%) |
| **499108** | 92/97 | USA (91%), Mexico (5%), Germany (2%), Israel (1%), Switzerland (1%) | North America (96%) | 2021-04-23 to 2021-08-17 | **Active** | AY.3 (100%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NSP3_P1228L (99%),NS7a_V82A (99%),Spike_T19R (98%),Spike_D950N (95%) |
| **499960** | 43/95 | United Kingdom (100%) | Europe (100%) | 2021-05-16 to 2021-08-21 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP14_A85S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T22I (100%) |
| **500093** | 209/830 | United Kingdom (99%), Japan (0%), USA (0%), Denmark (0%), France (0%) | Europe (100%) | 2021-04-26 to 2021-08-22 | **Active** | AY.4 (57%) | GK (98%) | Delta | Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (100%),NSP3_A1711V (100%),NS7a_T120I (100%),Spike_G142D (100%),N_G215C (99%) |
| **500916** | 75/170 | United Kingdom (98%), Portugal (2%), Italy (1%) | Europe (100%) | 2021-05-17 to 2021-08-22 | **Active** | B.1.617.2 (52%) | GK (96%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),Spike_T19R (99%),Spike_G142D (95%) |
| **503807** | 117/289 | United Kingdom (99%), Hong Kong (0%), USA (0%), Denmark (0%) | Europe (99%) | 2021-05-18 to 2021-08-22 | **Active** | B.1.617.2 (94%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_A55S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP8_T187I (94%),NSP2_T429I (94%),NSP13_T307I (93%) |
| **503816** | 52/133 | United Kingdom (100%) | Europe (100%) | 2021-05-18 to 2021-08-17 | **Active** | AY.11 (100%) | GK (99%) | Delta | NSP14_R163C (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A256V (100%),M_A2S (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),N_P20L (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP5_K236R (100%),N_R203M (99%),NSP2_S358L (65%) |
| **504040** | 74/196 | United Kingdom (100%) | Europe (100%) | 2021-05-18 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P251L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (100%) |
| **504211** | 84/305 | United Kingdom (100%) | Europe (100%) | 2021-05-18 to 2021-08-22 | **Active** | B.1.617.2 (56%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP12_H613Y (95%) |
| **504218** | 187/437 | United Kingdom (98%), Aruba (1%), USA (1%), Germany (0%) | Europe (99%) | 2021-05-18 to 2021-08-23 | **Active** | B.1.617.2 (63%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (100%),NSP12_P323L (100%),Spike_G142D (100%),NSP3_A1711V (99%) |
| **504636** | 24/34 | United Kingdom (97%), Indonesia (3%) | Europe (97%) | 2021-05-18 to 2021-08-22 | **Active** | AY.4 (76%) | GK (100%) | Delta | N_G215C (100%),Spike_V1264L (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (97%),NSP12_P323L (97%) |
| **504653** | 68/136 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-05-18 to 2021-08-19 | **Active** | B.1.617.2 (88%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),M_K15N (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%) |
| **505825** | 30/53 | United Kingdom (94%), Norway (4%), USA (2%) | Europe (98%) | 2021-05-19 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),N_P326L (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),Spike_T95I (98%),NSP12_E919D (66%) |
| **505914** | 59/290 | United Kingdom (99%), USA (1%), Portugal (0%) | Europe (99%) | 2021-05-19 to 2021-08-24 | **Active** | B.1.617.2 (81%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **506472** | 41/91 | United Kingdom (100%) | Europe (100%) | 2021-05-19 to 2021-08-22 | **Active** | B.1.617.2 (77%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **508788** | 74/140 | USA (97%), Colombia (2%), Germany (1%) | North America (97%) | 2021-04-08 to 2021-08-17 | **Active** | B.1.1.7 (100%) | GRY (71%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),NSP6_S106del (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),NSP3_T1063I (100%),NS8_C61F (100%),NSP12_P227L (100%),NSP6_G107del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NSP6_F108del (100%),NS8_Y73C (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NS8_K68stop (99%),Spike_Y144del (99%),NSP3_M1441I (99%),NSP3_S1807F (88%),NSP3_P662L (88%) |
| **508958** | 796/1309 | USA (96%), Mexico (3%), Japan (0%), Germany (0%), Costa Rica (0%) | North America (99%) | 2021-05-18 to 2021-08-22 | **Active** | AY.3 (100%) | GK (99%) | Delta | N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_G215C (100%),N_D63G (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_T478K (100%),Spike_L452R (100%),NSP6_T77A (99%),Spike_D950N (99%),Spike_G142D (94%) |
| **509326** | 441/788 | USA (99%), Canada (0%), Greece (0%), Hong Kong (0%), Denmark (0%) | North America (99%) | 2021-05-12 to 2021-08-18 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | NS7a_L116F (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D63G (100%),NSP14_S374F (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),Spike_P681R (100%),NSP2_P129L (100%),Spike_L452R (99%),Spike_D950N (98%),Spike_G142D (90%) |
| **509625** | 52/89 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-05-22 to 2021-08-23 | **Active** | B.1.617.2 (73%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%) |
| **509743** | 73/179 | United Kingdom (99%), Netherlands (1%) | Europe (100%) | 2021-05-22 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),NSP2_S378F (65%) |
| **510486** | 43/110 | United Kingdom (100%) | Europe (100%) | 2021-05-23 to 2021-08-21 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_G842S (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (100%) |
| **510589** | 157/306 | United Kingdom (95%), Netherlands (3%), Portugal (1%), Czech Republic (0%), Switzerland (0%) | Europe (100%) | 2021-05-23 to 2021-08-23 | **Active** | AY.4 (88%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **510598** | 57/164 | United Kingdom (100%) | Europe (100%) | 2021-05-23 to 2021-08-22 | **Active** | B.1.617.2 (95%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **510976** | 74/215 | United Kingdom (99%), Italy (0%), Switzerland (0%), USA (0%) | Europe (100%) | 2021-05-24 to 2021-08-22 | **Active** | AY.4 (49%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),Spike_G142D (99%),NSP3_T611I (73%),NSP2_L451F (73%) |
| **510979** | 401/879 | United Kingdom (96%), USA (3%), Germany (0%), Hong Kong (0%), Bangladesh (0%) | Europe (97%) | 2021-05-24 to 2021-08-23 | **Active** | AY.4 (64%) | GK (99%) | Delta | Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),NSP3_A1711V (100%),Spike_D950N (100%),Spike_D138H (100%) |
| **511010** | 25/98 | United Kingdom (100%) | Europe (100%) | 2021-05-24 to 2021-08-21 | **Active** | B.1.617.2 (80%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP3_V965F (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **511018** | 101/305 | United Kingdom (99%), USA (0%), Spain (0%), Belgium (0%) | Europe (100%) | 2021-05-14 to 2021-08-22 | **Active** | B.1.617.2 (65%) | GK (96%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),NSP3_E1091A (79%) |
| **511054** | 44/82 | United Kingdom (98%), USA (2%) | Europe (98%) | 2021-05-24 to 2021-08-21 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),N_H145Y (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **511073** | 63/128 | United Kingdom (100%) | Europe (100%) | 2021-05-24 to 2021-08-22 | **Active** | B.1.617.2 (77%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T19R (99%),NSP6_L37F (99%) |
| **512274** | 663/970 | USA (99%), Sint Maarten (0%), Mexico (0%), Netherlands (0%), United Kingdom (0%) | North America (99%) | 2021-05-18 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_D63G (100%),M_S197N (100%),N_R203M (100%),NSP12_G671S (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_A222V (100%),N_D377Y (100%),Spike_E156G (100%),NSP6_V149A (100%),NSP3_P822L (100%),NS3_G224C (100%),Spike_D950N (99%),Spike_G142D (90%) |
| **512475** | 91/134 | United Kingdom (80%), USA (19%), France (1%), Japan (1%) | Europe (81%) | 2021-05-24 to 2021-08-22 | **Active** | B.1.617.2 (70%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_L5F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (99%),NSP6_T77A (99%),NSP3_A1711V (79%) |
| **512557** | 122/256 | United Kingdom (100%), Romania (0%) | Europe (100%) | 2021-05-25 to 2021-08-23 | **Active** | AY.4 (76%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_D950N (100%),NSP12_P323L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **513532** | 82/126 | USA (99%), Denmark (1%) | North America (99%) | 2021-05-22 to 2021-08-20 | **Active** | B.1.617.2 (94%) | GK (97%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_K1191N (100%),NSP6_T181I (100%),NS3_G100V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),NSP3_T1022I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (99%),Spike_D950N (98%),Spike_G142D (83%) |
| **514412** | 41/165 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-05-26 to 2021-08-18 | **Active** | AY.4 (79%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP2_A302V (97%),NSP13_P504L (97%) |
| **514788** | 26/56 | United Kingdom (100%) | Europe (100%) | 2021-05-26 to 2021-08-19 | **Active** | B.1.617.2 (91%) | GK (88%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_A845S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **515416** | 631/1286 | United Kingdom (94%), Denmark (3%), Germany (1%), France (1%), Switzerland (0%) | Europe (100%) | 2021-05-26 to 2021-08-23 | **Active** | B.1.617.2 (68%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (100%),Spike_P681R (100%),Spike_T95I (100%),NS3_K235T (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_G142D (96%) |
| **515443** | 38/67 | Spain (39%), Italy (31%), France (12%), United Kingdom (10%), Germany (4%) | Europe (100%) | 2021-05-25 to 2021-08-24 | **Active** | B.1.617.2 (99%) | GK (96%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP6_T181I (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_S1252F (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),NSP13_H164Y (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP14_I298V (99%),NS3_S26L (99%),NS7a_V82A (99%),Spike_T19R (99%),M_I82T (99%),Spike_D950N (99%),Spike_T478K (99%) |
| **515730** | 227/477 | United Kingdom (99%), Netherlands (1%), Norway (0%), Japan (0%) | Europe (100%) | 2021-05-27 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),N_G96C (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (100%),Spike_P681R (100%) |
| **515763** | 44/86 | United Kingdom (100%) | Europe (100%) | 2021-05-27 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (99%),NSP15_A137T (71%) |
| **515848** | 27/62 | United Kingdom (100%) | Europe (100%) | 2021-05-27 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (95%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T678I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **515898** | 75/121 | USA (97%), United Kingdom (2%), Norway (1%), South Korea (1%) | North America (97%) | 2021-05-22 to 2021-08-21 | **Active** | B.1.617.2 (83%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),Spike_H1101Y (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1228L (99%),Spike_D950N (99%),N_T362I (98%),Spike_T95I (97%),Spike_P681R (97%),NSP3_P1469S (92%),Spike_G142D (83%) |
| **516487** | 41/118 | United Kingdom (100%) | Europe (100%) | 2021-05-27 to 2021-08-23 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_L41F (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_A65S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (99%) |
| **516494** | 89/135 | Portugal (76%), Italy (17%), Belgium (2%), Germany (1%), United Kingdom (1%) | Europe (100%) | 2021-05-27 to 2021-08-17 | **Active** | AY.12 (93%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),NSP16_P215T (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_S263F (99%),Spike_D950N (99%) |
| **516969** | 114/184 | Spain (38%), Switzerland (14%), Germany (11%), Italy (8%), France (8%) | Europe (99%) | 2021-05-28 to 2021-08-18 | **Active** | AY.12 (91%) | GK (98%) | Delta | Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (99%),NS7a_V82A (99%),N_D377Y (99%),NSP6_T77A (99%),Spike_D950N (99%),NSP4_V167L (99%),N_D63G (98%),NSP2_V485I (83%) |
| **517143** | 1641/2386 | USA (99%), Mexico (0%), United Kingdom (0%), Japan (0%), Hong Kong (0%) | North America (99%) | 2021-04-23 to 2021-08-21 | **Active** | AY.3 (86%) | GK (99%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),N_D377Y (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_R203M (100%),NS7a_V82A (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP4_V167L (100%),Spike_T19R (100%),NSP3_P1469S (100%),Spike_L452R (100%),N_G215C (100%),N_D63G (100%),NSP6_I162V (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),NSP6_T77A (100%),Spike_D950N (99%),Spike_G142D (88%) |
| **517387** | 64/85 | Portugal (80%), France (15%), United Kingdom (2%), Belgium (1%), Switzerland (1%) | Europe (100%) | 2021-05-28 to 2021-08-17 | **Active** | AY.12 (82%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS3_G224C (100%),Spike_T478K (100%),Spike_Q218H (100%),NSP12_V257F (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),NSP4_V167L (100%),NSP6_L37F (100%),Spike_L452R (100%),NSP12_G671S (99%),Spike_D950N (99%),NSP13_P77L (99%) |
| **517549** | 46/184 | United Kingdom (99%), France (1%), Germany (1%) | Europe (100%) | 2021-05-29 to 2021-08-20 | **Active** | B.1.617.2 (86%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (99%),NSP14_T16I (98%),NSP12_R197Q (98%) |
| **517556** | 77/169 | United Kingdom (83%), Italy (16%), France (1%) | Europe (100%) | 2021-05-12 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_T19R (99%),Spike_D950N (99%),NS7a_T120I (98%),NSP3_A1941T (95%),Spike_G142D (92%) |
| **517659** | 48/146 | United Kingdom (99%), Switzerland (1%) | Europe (100%) | 2021-05-25 to 2021-08-22 | **Active** | B.1.617.2 (68%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NS7a_G38stop (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),Spike_T95I (99%),Spike_G142D (99%) |
| **517759** | 71/143 | United Kingdom (91%), USA (5%), Denmark (1%), Bosnia and Herzegovina (1%), India (1%) | Europe (94%) | 2021-04-21 to 2021-08-22 | **Active** | B.1.617.2 (73%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS8_W45L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),Spike_D950N (99%),Spike_T95I (96%),NSP3_A1711V (93%) |
| **518540** | 70/166 | United Kingdom (95%), Denmark (3%), USA (2%) | Europe (98%) | 2021-05-30 to 2021-08-23 | **Active** | B.1.617.2 (64%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NS7a_T120I (99%),NSP3_A1711V (98%) |
| **518788** | 50/86 | United Kingdom (100%) | Europe (100%) | 2021-05-29 to 2021-08-21 | **Active** | B.1.617.2 (62%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_K235T (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP4_N115S (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **519016** | 110/338 | United Kingdom (80%), Greece (10%), Netherlands (3%), Denmark (2%), Italy (1%) | Europe (100%) | 2021-05-21 to 2021-08-19 | **Active** | B.1.617.2 (92%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (98%),NSP14_D324Y (97%) |
| **519136** | 162/353 | United Kingdom (99%), India (0%), USA (0%), Romania (0%), Germany (0%) | Europe (99%) | 2021-05-12 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),Spike_T95I (100%),NS7a_V82A (100%),Spike_D950N (100%),NS7a_L5P (99%),Spike_T19R (99%),Spike_G142D (99%),NSP2_T634I (99%),NS7a_T120I (98%) |
| **519201** | 75/188 | United Kingdom (97%), Italy (2%), Romania (1%) | Europe (100%) | 2021-05-31 to 2021-08-22 | **Active** | AY.4 (65%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NS7a_H73Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP12_D879N (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP5_K88R (99%),Spike_G142D (96%) |
| **519328** | 38/82 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-01 to 2021-08-19 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_A1020V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%) |
| **519551** | 21/39 | United Kingdom (100%) | Europe (100%) | 2021-06-01 to 2021-08-22 | **Active** | AY.4 (69%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_R158del (100%),NS8_V62M (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_P681R (97%),NSP3_E1091A (67%) |
| **519602** | 43/87 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-01 to 2021-08-20 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | Spike_R21I (100%),N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%) |
| **519773** | 193/448 | United Kingdom (93%), Italy (2%), Denmark (2%), Switzerland (1%), France (1%) | Europe (100%) | 2021-06-01 to 2021-08-22 | **Active** | AY.9 (52%) | GK (98%) | Delta | NS3_A23S (100%),N_R203M (100%),NSP6_T181I (100%),NS3_S26L (100%),NSP2_T528I (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP9_T21I (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP12_G671S (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_D950N (100%),N_D63G (99%),Spike_G142D (92%) |
| **519836** | 158/199 | USA (99%), Monaco (1%) | North America (99%) | 2021-06-01 to 2021-08-18 | **Active** | AY.3 (58%) | GK (96%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_T478K (99%),Spike_L452R (99%),Spike_D950N (98%) |
| **519948** | 167/303 | United Kingdom (100%) | Europe (100%) | 2021-06-01 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **519993** | 42/111 | United Kingdom (90%), USA (5%), Sweden (2%), Netherlands (1%), France (1%) | Europe (94%) | 2021-06-01 to 2021-08-22 | **Active** | B.1.617.2 (89%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NS7a_V82A (99%),NS7a_T120I (99%),NS7b_T40I (97%),NSP3_A1711V (94%),NSP2_A159V (82%) |
| **520014** | 26/43 | United Kingdom (67%), USA (23%), Portugal (2%), Belgium (2%), Italy (2%) | Europe (77%) | 2021-04-26 to 2021-08-19 | **Active** | B.1.617.2 (77%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS3_A54S (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP13_P77L (98%),Spike_G142D (95%),Spike_T95I (77%),NSP3_A1711V (70%) |
| **520155** | 93/138 | United Kingdom (94%), Czech Republic (2%), Sweden (1%), Germany (1%), USA (1%) | Europe (99%) | 2021-05-25 to 2021-08-22 | **Active** | B.1.617.2 (74%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_F86del (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS8_T87S (100%),NSP3_A1711V (99%),Spike_T95I (99%),Spike_G142D (99%) |
| **520175** | 170/424 | United Kingdom (71%), Denmark (13%), Turkey (9%), Germany (3%), Switzerland (1%) | Europe (99%) | 2021-06-02 to 2021-08-24 | **Active** | B.1.617.2 (71%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),N_S232G (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (97%) |
| **520267** | 51/201 | United Kingdom (100%), Sweden (0%) | Europe (100%) | 2021-06-02 to 2021-08-22 | **Active** | B.1.617.2 (80%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7b_T40I (100%) |
| **520348** | 172/386 | United Kingdom (97%), USA (1%), Mexico (1%), Italy (0%), Indonesia (0%) | Europe (98%) | 2021-05-10 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (100%),Spike_G142D (99%),Spike_T95I (99%),NSP3_A1711V (98%),NSP14_A85S (95%) |
| **520547** | 131/227 | USA (79%), United Kingdom (19%), France (1%), Italy (0%), Japan (0%) | North America (79%) | 2021-05-03 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NS3_S26L (100%),Spike_P681R (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_G671S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_P1469S (100%),Spike_F157del (99%),Spike_D950N (99%),Spike_E156G (97%),NSP12_P323L (97%),Spike_G142D (93%),NSP12_R249M (78%),NSP2_W466L (78%) |
| **520965** | 134/265 | United Kingdom (99%), Canada (0%), Japan (0%), USA (0%) | Europe (99%) | 2021-06-03 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),Spike_A67V (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),Spike_G142D (99%),NSP14_P412S (92%) |
| **521091** | 33/80 | United Kingdom (91%), Portugal (8%), USA (1%) | Europe (99%) | 2021-06-03 to 2021-08-21 | **Active** | B.1.617.2 (82%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (92%),Spike_G142D (92%),NSP3_A1711V (91%) |
| **521219** | 68/202 | United Kingdom (100%), Portugal (0%) | Europe (100%) | 2021-06-03 to 2021-08-22 | **Active** | B.1.617.2 (69%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP14_D324N (100%),Spike_G142D (100%),Spike_V70F (87%) |
| **521254** | 48/80 | United Kingdom (84%), USA (4%), Denmark (4%), Brazil (2%), Switzerland (1%) | Europe (92%) | 2021-06-03 to 2021-08-21 | **Active** | B.1.617.2 (94%) | GK (95%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (96%),Spike_T95I (75%) |
| **521345** | 90/212 | United Kingdom (100%) | Europe (100%) | 2021-05-28 to 2021-08-23 | **Active** | B.1.617.2 (50%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_S1252F (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP13_T599I (89%),NSP13_K276N (89%) |
| **521388** | 27/39 | United Kingdom (100%) | Europe (100%) | 2021-04-29 to 2021-08-22 | **Active** | AY.4 (62%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_V273L (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (97%) |
| **521391** | 281/581 | United Kingdom (97%), Netherlands (1%), Denmark (1%), Norway (0%), Sweden (0%) | Europe (100%) | 2021-06-03 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (100%),NSP13_P77L (100%) |
| **521498** | 76/165 | United Kingdom (99%), Norway (1%) | Europe (100%) | 2021-06-04 to 2021-08-22 | **Active** | AY.4 (73%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP16_A168V (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS7a_K32N (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **521708** | 72/158 | United Kingdom (99%), Japan (1%) | Europe (99%) | 2021-06-04 to 2021-08-22 | **Active** | B.1.617.2 (61%) | GK (99%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (99%),NSP13_P77L (99%),NSP6_T77A (99%),NSP14_D324N (99%) |
| **522062** | 28/49 | United Kingdom (98%), Italy (2%) | Europe (100%) | 2021-06-05 to 2021-08-22 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_S704L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),NSP2_N98T (84%) |
| **522120** | 23/30 | United Kingdom (97%), Netherlands (3%) | Europe (100%) | 2021-05-16 to 2021-08-20 | **Active** | B.1.617.2 (60%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),Spike_D574Y (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (97%) |
| **522196** | 156/360 | United Kingdom (98%), USA (1%), Poland (0%), Greece (0%), Netherlands (0%) | Europe (99%) | 2021-06-06 to 2021-08-23 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),Spike_D950N (100%),Spike_T95I (99%),NSP3_S1230F (98%) |
| **522271** | 77/173 | United Kingdom (99%), Japan (1%) | Europe (99%) | 2021-06-06 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_G181E (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_V965F (98%) |
| **522393** | 40/110 | United Kingdom (95%), Denmark (2%), Netherlands (1%), Germany (1%), Norway (1%) | Europe (100%) | 2021-06-06 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (99%),NSP3_A1711V (95%),Spike_T95I (95%) |
| **522532** | 89/275 | United Kingdom (93%), France (2%), Spain (1%), Norway (1%), Denmark (1%) | Europe (98%) | 2021-04-26 to 2021-08-23 | **Active** | AY.4 (65%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (100%),Spike_G142D (98%),NSP3_A1711V (98%),NSP2_G481S (92%) |
| **522647** | 25/37 | United Kingdom (78%), Belgium (16%), Spain (3%), USA (3%) | Europe (97%) | 2021-06-07 to 2021-08-21 | **Active** | B.1.617.2 (86%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (97%),Spike_G142D (92%),NSP3_A1711V (89%) |
| **522698** | 53/96 | United Kingdom (100%) | Europe (100%) | 2021-05-19 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (95%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_P207S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_G142D (97%),NSP3_P1469S (96%) |
| **522733** | 75/151 | United Kingdom (95%), Italy (4%), USA (1%) | Europe (99%) | 2021-06-07 to 2021-08-23 | **Active** | B.1.617.2 (75%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),Spike_T95I (99%),Spike_G142D (99%),NSP16_S276I (60%),NSP14_S418I (60%) |
| **522932** | 92/155 | United Kingdom (99%), USA (1%), Denmark (1%) | Europe (99%) | 2021-05-26 to 2021-08-22 | **Active** | B.1.617.2 (81%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T478K (99%),NS3_E239D (99%),NSP3_A1941V (92%) |
| **522987** | 605/937 | Germany (19%), Portugal (16%), Turkey (13%), United Kingdom (11%), USA (10%) | Europe (84%) | 2021-04-04 to 2021-08-24 | **Active** | AY.12 (48%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),Spike_T478K (100%),Spike_D614G (100%),Spike_T95I (100%),N_R203M (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NS7a_T120I (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP12_G671S (100%),NSP3_P1228L (100%),NSP13_P77L (100%),NSP4_T492I (99%),Spike_L452R (99%),NSP3_P1469S (99%),NSP14_A394V (99%),NSP12_P323L (98%),NSP3_A488S (98%),NSP6_T77A (98%),NSP4_V167L (90%) |
| **523005** | 28/48 | United Kingdom (98%), Portugal (2%) | Europe (100%) | 2021-06-02 to 2021-08-23 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),Spike_G142D (98%),NSP3_T611I (88%),NSP2_L451F (88%) |
| **523085** | 618/1384 | United Kingdom (99%), Germany (1%), Denmark (0%), Italy (0%), France (0%) | Europe (100%) | 2021-05-23 to 2021-08-24 | **Active** | AY.4 (71%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),NS7a_V82A (100%),NSP14_A394V (100%),NS7a_T120I (100%),Spike_D950N (100%),Spike_T19R (100%),Spike_G142D (100%),Spike_A222V (97%),Spike_Y145H (97%) |
| **523144** | 29/45 | United Kingdom (93%), USA (4%), Netherlands (2%) | Europe (96%) | 2021-06-08 to 2021-08-22 | **Active** | B.1.617.2 (80%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),N_T141I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%),NSP3_A1711V (93%) |
| **523158** | 26/46 | United Kingdom (100%) | Europe (100%) | 2021-06-08 to 2021-08-21 | **Active** | AY.4 (74%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),N_T417I (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1469S (98%),NSP6_T77A (98%) |
| **523274** | 51/102 | United Kingdom (98%), France (1%), USA (1%) | Europe (99%) | 2021-06-08 to 2021-08-23 | **Active** | B.1.617.2 (85%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),NSP3_A1711V (98%) |
| **523323** | 31/67 | United Kingdom (97%), Portugal (3%) | Europe (100%) | 2021-06-06 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (97%),Spike_T95I (97%),Spike_G142D (97%),NSP2_D155N (97%) |
| **523339** | 29/51 | United Kingdom (96%), Denmark (2%), USA (2%) | Europe (98%) | 2021-06-09 to 2021-08-23 | **Active** | B.1.617.2 (94%) | GK (96%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS6_V24F (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),NSP3_M1562I (78%) |
| **523396** | 30/38 | United Kingdom (97%), Kuwait (3%) | Europe (97%) | 2021-06-09 to 2021-08-22 | **Active** | B.1.617.2 (84%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (95%) |
| **523404** | 497/881 | Denmark (18%), Netherlands (14%), France (14%), Italy (10%), United Kingdom (8%) | Europe (90%) | 2021-05-06 to 2021-08-24 | **Active** | B.1.617.2 (96%) | GK (99%) | Delta | NSP12_G671S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (100%),Spike_P251L (100%),N_D377Y (100%),NSP4_V167L (100%),NS3_S26L (100%),NS7a_V82A (100%),Spike_T19R (100%),NSP13_P77L (100%),NSP3_E906D (100%),Spike_T478K (100%),NSP1_E87D (100%),N_R203M (99%),N_D63G (99%),NS8_A65S (99%),Spike_D950N (95%),Spike_G142D (85%) |
| **523424** | 68/121 | United Kingdom (93%), USA (6%), Mexico (1%) | Europe (93%) | 2021-06-09 to 2021-08-23 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),Spike_G1219V (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),NSP6_T77A (98%),NSP1_E87D (93%) |
| **523531** | 44/238 | United Kingdom (100%) | Europe (100%) | 2021-06-02 to 2021-08-19 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **523600** | 48/105 | United Kingdom (100%) | Europe (100%) | 2021-06-10 to 2021-08-19 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (95%) |
| **523629** | 53/101 | United Kingdom (99%), Hong Kong (1%) | Europe (99%) | 2021-05-20 to 2021-08-18 | **Active** | B.1.617.2 (87%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP3_P723L (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **538268** | 152/206 | France (59%), Switzerland (13%), Germany (11%), Denmark (3%), Netherlands (2%) | Europe (98%) | 2021-02-22 to 2021-08-25 | **Active** | B.1.617.2 (49%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_A318V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_G181V (100%),NSP6_T77A (100%),NS7a_R118G (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (98%),Spike_D950N (95%) |
| **551148** | 24/36 | United Kingdom (50%), USA (14%), Sint Maarten (11%), Poland (8%), Switzerland (6%) | Europe (67%) | 2021-01-07 to 2021-08-18 | **Active** | B.1.1.7 (100%) | GRY (81%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),NSP6_S106del (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP6_G107del (100%),NSP3_A890D (100%),NS7a_G38V (100%),Spike_D1118H (100%),NSP6_F108del (100%),NS8_Y73C (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%) |
| **551249** | 247/321 | USA (70%), Dominican Republic (10%), Spain (8%), Switzerland (3%), Belgium (2%) | North America (82%) | 2021-04-23 to 2021-08-20 | **Active** | B.1.621.1 (98%) | GH (100%) | Mu | Spike_E484K (100%),NSP3_T720I (100%),Spike_N501Y (100%),Spike_P681H (100%),NSP4_T492I (100%),NSP6_Q160R (100%),Spike_R346K (100%),N_T205I (100%),NS3_Q57H (100%),NSP3_T237A (100%),NSP12_P323L (100%),Spike_D614G (100%),NS8_P38S (100%),NSP13_P419S (100%),NS8_T11K (100%),NSP15_S261L (99%),NSP4_T189I (99%),NSP13_P491S (99%),NSP3_N1329D (99%),Spike_T95I (99%),NS3_V256I (98%),Spike_D950N (97%),NS3_N257H (95%),Spike_Y144T (90%),Spike_ins145N (89%),Spike_Y145S (88%),NS3_V259L (84%),NS8_S67F (82%) |
| **551777** | 162/323 | USA (100%) | North America (100%) | 2021-04-27 to 2021-08-18 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP14_S374F (100%),Spike_F157del (100%),Spike_T19R (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D377Y (100%),Spike_L452R (100%),NS7a_V82A (99%),Spike_D950N (99%),NS7a_T120I (98%),NS7a_L116F (98%),Spike_G142D (80%) |
| **556538** | 597/1150 | France (22%), Switzerland (17%), Italy (14%), Spain (14%), Germany (13%) | Europe (100%) | 2021-05-06 to 2021-08-24 | **Active** | AY.4 (96%) | GK (98%) | Delta | N_G215C (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_R203M (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP3_P1469S (100%),M_I82T (100%),NS7a_V82A (99%),NSP14_A394V (99%),NSP4_V167L (99%),Spike_D950N (99%),NSP12_L838I (98%),N_D63G (97%) |
| **559045** | 365/577 | USA (99%), Mexico (1%) | North America (100%) | 2021-04-28 to 2021-08-17 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),Spike_D614G (100%),Spike_L452R (100%),NSP2_P129L (100%),NSP3_P822L (100%),NSP12_P323L (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_D950N (97%),Spike_G142D (84%),NSP5_P184H (68%) |
| **560466** | 170/257 | USA (98%), United Kingdom (2%) | North America (98%) | 2021-05-12 to 2021-08-19 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (100%),Spike_D950N (100%),NSP3_P1469S (100%),NSP13_I334V (99%),NSP2_G339S (98%),NS7a_T120I (98%),Spike_G142D (92%) |
| **564909** | 77/108 | USA (99%), Japan (1%) | North America (99%) | 2021-05-19 to 2021-08-17 | **Active** | B.1.617.2 (85%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_R214H (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP12_F192V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (99%),Spike_G142D (85%) |
| **565702** | 168/233 | Indonesia (67%), South Korea (15%), Switzerland (8%), France (5%), USA (2%) | Asia (82%) | 2021-03-25 to 2021-08-19 | **Active** | AY.4 (95%) | GK (94%) | Delta | N_G215C (100%),Spike_V1264L (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NS7a_V82A (100%),Spike_L452R (100%),NSP3_P1228L (99%),N_D63G (99%),Spike_D950N (98%),NSP3_T678I (98%),NSP2_P200L (98%) |
| **566515** | 160/329 | Netherlands (74%), Curacao (8%), Germany (5%), Sint Maarten (5%), Denmark (2%) | Europe (84%) | 2021-05-21 to 2021-08-18 | **Active** | B.1.617.2 (93%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP5_P132L (99%),NSP14_T16I (99%),NSP12_R197Q (99%),Spike_A67V (99%),Spike_D950N (99%),NSP3_V61I (99%),Spike_G142D (94%) |
| **571291** | 99/147 | Bahrain (56%), Turkey (25%), USA (5%), France (3%), Belgium (3%) | Asia (59%) | 2021-04-10 to 2021-08-24 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_R203M (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_L452R (100%),Spike_P681R (99%),Spike_D614G (99%),NSP12_G671S (99%),NSP3_P822L (99%),NSP13_P77L (99%),NSP4_A446V (99%),Spike_D950N (97%),Spike_V1264L (96%),Spike_A222V (95%),NSP12_M196I (95%),N_D63G (95%),NSP6_T181I (94%),NSP6_V149A (94%) |
| **573348** | 224/296 | USA (66%), Mexico (29%), France (2%), Denmark (1%), United Kingdom (1%) | North America (95%) | 2021-06-01 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | Spike_V1264L (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS6_K48N (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_P681R (100%),NSP6_V149A (100%),NSP3_P822L (100%),Spike_D950N (99%),Spike_G142D (99%),Spike_N1074S (95%) |
| **573690** | 897/1487 | USA (99%), Aruba (1%), Japan (0%), Mexico (0%), Norway (0%) | North America (99%) | 2021-06-01 to 2021-08-21 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP3_A488S (100%),NSP4_T492I (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),N_D63G (100%),NS7b_T40I (100%),NSP12_P323L (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_E156G (100%),NSP13_I334V (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_L452R (100%),N_R203M (100%),Spike_P681R (100%),NSP3_P1469S (100%),NSP6_T77A (99%),Spike_D950N (98%),Spike_G142D (90%) |
| **573975** | 322/746 | France (28%), Italy (25%), United Kingdom (9%), Switzerland (7%), Finland (7%) | Europe (96%) | 2021-06-01 to 2021-08-20 | **Active** | AY.4 (95%) | GK (98%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (100%),N_D377Y (100%),N_Q9L (100%),NSP3_P1469S (100%),NSP4_V167L (100%),NSP3_A488S (100%),NSP14_A394V (100%),NS7b_T40I (100%),N_D63G (92%) |
| **574076** | 129/224 | Portugal (87%), Switzerland (3%), United Kingdom (2%), Spain (2%), Sweden (2%) | Europe (100%) | 2021-05-19 to 2021-08-17 | **Active** | AY.4 (92%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP13_V247F (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP3_Q1216H (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%) |
| **574257** | 52/141 | Turkey (43%), Netherlands (25%), Norway (6%), USA (6%), Sweden (5%) | Europe (91%) | 2021-04-26 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_L452R (100%),Spike_G142D (99%),Spike_P681R (99%),Spike_D950N (99%),Spike_D614G (99%),NS7a_V82A (99%),NSP6_T181I (92%),NSP3_I1723V (88%),NSP12_D269G (79%) |
| **574656** | 66/112 | USA (99%), Japan (1%) | North America (99%) | 2021-06-02 to 2021-08-19 | **Active** | AY.1 (96%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),M_L29F (100%),NSP12_G671S (100%),Spike_K417N (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP2_K489N (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_W258L (99%),NSP6_T77A (97%),NSP3_P1228L (96%),NSP14_A394V (96%),NS3_L53F (96%),Spike_D950N (96%),Spike_G142D (88%) |
| **576218** | 41/65 | Portugal (91%), Spain (3%), Switzerland (3%), Belgium (2%), Germany (2%) | Europe (100%) | 2021-06-04 to 2021-08-17 | **Active** | AY.5 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),M_N207K (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **576306** | 193/395 | Japan (99%), Hong Kong (0%), USA (0%), United Kingdom (0%), New Zealand (0%) | Asia (99%) | 2021-05-30 to 2021-08-19 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS8_P93S (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_V932A (100%),NSP14_A394V (100%),Spike_G142D (98%),N_D377Y (96%) |
| **577604** | 118/219 | United Kingdom (95%), Turkey (2%), Finland (1%), France (0%), USA (0%) | Europe (100%) | 2021-06-06 to 2021-08-23 | **Active** | B.1.617.2 (89%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),Spike_G446V (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (100%),Spike_D950N (100%),N_D63G (99%),NSP3_A1711V (97%),Spike_G142D (97%),NSP15_I252L (81%),NSP2_T634I (81%) |
| **578239** | 61/221 | United Kingdom (99%), Spain (1%) | Europe (100%) | 2021-05-28 to 2021-08-22 | **Active** | B.1.617.2 (86%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (100%),Spike_G142D (99%),NSP10_R134C (89%) |
| **578525** | 76/108 | Portugal (94%), Switzerland (4%), Spain (1%), United Kingdom (1%) | Europe (100%) | 2021-06-07 to 2021-08-17 | **Active** | AY.12 (98%) | GK (98%) | Delta | Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS3_A110S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (99%),N_D63G (99%),NS7a_T120I (99%),NSP5_K90R (99%),NSP16_P215T (99%),NSP12_P323L (98%),NSP2_S263F (97%) |
| **580005** | 61/147 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-05-24 to 2021-08-22 | **Active** | B.1.617.2 (95%) | GK (100%) | Delta | N_G215C (100%),NS3_A33S (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_T19R (99%),Spike_T95I (94%),NSP3_A1711V (93%),NSP6_Q160K (87%),NSP2_T634I (87%) |
| **580013** | 65/98 | United Kingdom (90%), USA (4%), Turkey (3%), Netherlands (1%), Japan (1%) | Europe (95%) | 2021-06-08 to 2021-08-22 | **Active** | AY.4 (88%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP3_P1228L (99%),NSP3_A488S (99%),NSP4_T492I (99%),NSP14_A394V (99%),Spike_D950N (99%),NS7b_T40I (99%),NSP3_P1469S (99%),NSP6_T77A (99%),NSP4_V167L (96%),Spike_G142D (95%),Spike_T95I (92%),NSP3_A1711V (85%) |
| **580111** | 68/121 | Belgium (68%), Netherlands (22%), Aruba (6%), Germany (2%), Denmark (1%) | Europe (94%) | 2021-05-29 to 2021-08-19 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),N_P207S (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_K478N (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP3_G730S (97%),NS7a_V24F (96%),Spike_G142D (88%) |
| **580301** | 53/95 | Portugal (100%) | Europe (100%) | 2021-06-08 to 2021-08-17 | **Active** | AY.12 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),Spike_L8V (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),NSP16_P215T (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_S263F (98%) |
| **580500** | 105/305 | USA (78%), Canada (22%), Sweden (0%), Costa Rica (0%) | North America (100%) | 2021-06-08 to 2021-08-18 | **Active** | B.1.617.2 (88%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),Spike_D950N (99%),NSP6_C197F (97%),NSP3_D410G (92%),Spike_G142D (88%) |
| **581015** | 86/134 | USA (93%), United Kingdom (7%), Mexico (1%) | North America (93%) | 2021-06-07 to 2021-08-21 | **Active** | AY.3 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP3_A1798V (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_G142D (80%) |
| **581021** | 69/144 | USA (100%) | North America (100%) | 2021-06-08 to 2021-08-17 | **Active** | AY.3 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_D950N (99%),Spike_G142D (80%) |
| **581024** | 1514/2464 | USA (100%), United Kingdom (0%), Sint Maarten (0%), Brazil (0%) | North America (100%) | 2021-05-03 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | NS3_S26L (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),Spike_E156G (100%),N_G18V (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_A222V (100%),NSP6_V149A (100%),NSP4_A446V (100%),Spike_L452R (100%),N_D63G (100%),Spike_V289I (100%),NSP6_T181I (100%),NSP3_P822L (100%),NSP3_E815D (99%),Spike_D950N (99%),Spike_G142D (92%) |
| **581254** | 85/285 | France (28%), Spain (25%), Switzerland (15%), Germany (12%), Italy (9%) | Europe (100%) | 2021-06-09 to 2021-08-18 | **Active** | AY.4 (98%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP14_A394V (100%),Spike_D950N (100%),NSP3_P1469S (100%),N_D63G (96%) |
| **581489** | 42/60 | USA (42%), Brazil (18%), Mexico (17%), Belgium (15%), Spain (3%) | North America (58%) | 2021-04-27 to 2021-08-19 | **Active** | B.1.617.2 (93%) | GK (88%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (98%),Spike_T95I (97%),Spike_D950N (93%) |
| **581579** | 38/77 | United Kingdom (99%), Norway (1%) | Europe (100%) | 2021-06-09 to 2021-08-22 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),Spike_T95I (99%) |
| **582236** | 73/106 | USA (100%) | North America (100%) | 2021-06-02 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NS3_G224C (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),M_S197N (99%),Spike_G142D (85%) |
| **582482** | 27/96 | USA (100%) | North America (100%) | 2021-06-09 to 2021-08-19 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_S358A (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP14_S374F (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP16_M20I (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (69%) |
| **582525** | 191/279 | France (34%), Germany (21%), Switzerland (15%), Spain (14%), Italy (7%) | Europe (100%) | 2021-05-03 to 2021-08-21 | **Active** | AY.12 (97%) | GK (97%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_T95I (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_E156G (100%),NSP4_V167L (100%),Spike_T719I (99%),NSP14_A394V (97%),N_D63G (96%),Spike_D950N (91%),NSP2_V485I (68%) |
| **582953** | 469/1072 | Netherlands (73%), Denmark (7%), Belgium (6%), Spain (4%), Germany (3%) | Europe (99%) | 2021-04-17 to 2021-08-19 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_R203M (100%),NS3_S26L (100%),N_D377Y (100%),M_I82T (100%),Spike_T478K (100%),Spike_D614G (100%),NSP12_G671S (100%),Spike_P681R (100%),NSP13_P77L (100%),NSP12_P323L (100%),Spike_L452R (100%),N_D63G (100%),NSP6_T181I (100%),NS7a_V82A (100%),Spike_T19R (100%),NSP4_A446V (100%),N_R209I (100%),NS7a_T120I (100%),N_M210I (100%),NSP6_V149A (100%),Spike_R21T (100%),NSP3_P822L (100%),Spike_A222V (99%),Spike_D950N (99%),Spike_F157del (99%),Spike_R158del (99%),Spike_E156G (99%),Spike_G142D (94%),NSP5_K90R (93%),NSP1_F143del (93%),NSP1_K141del (93%),NSP1_S142del (93%),NSP3_C1071Y (72%) |
| **583018** | 93/156 | Portugal (65%), Sweden (10%), Netherlands (8%), United Kingdom (4%), Gibraltar (3%) | Europe (97%) | 2021-06-07 to 2021-08-22 | **Active** | AY.12 (99%) | GK (99%) | Delta | NSP12_P323L (100%),Spike_D614G (100%),N_G215C (99%),Spike_T95I (99%),N_D63G (99%),N_R203M (99%),NSP12_G671S (99%),NS3_S26L (99%),Spike_P681R (99%),NS3_A110S (99%),Spike_R158del (99%),NSP3_P1228L (99%),NS7a_V82A (99%),NSP3_A488S (99%),Spike_F157del (99%),NSP4_T492I (99%),NSP14_A394V (99%),Spike_T19R (99%),NS7a_T120I (99%),N_D377Y (99%),M_I82T (99%),Spike_D950N (99%),NS7b_T40I (99%),NSP13_P77L (99%),Spike_E156G (99%),NSP3_P1469S (99%),Spike_T478K (99%),NSP6_T77A (99%),NSP5_K90R (99%),NSP16_P215T (99%),NSP4_V167L (99%),Spike_L452R (99%),NSP2_S263F (97%) |
| **583042** | 41/96 | Sweden (59%), Portugal (12%), Denmark (9%), Netherlands (5%), United Kingdom (3%) | Europe (100%) | 2021-06-10 to 2021-08-19 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP13_V247F (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP3_Q1216H (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_T19R (98%),N_D402Y (64%) |
| **583101** | 35/60 | USA (100%) | North America (100%) | 2021-05-25 to 2021-08-17 | **Active** | B.1.617.2 (98%) | GK (97%) | Delta | Spike_V1264L (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS6_K48N (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_R80I (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (97%) |
| **583812** | 35/125 | United Kingdom (98%), USA (1%), Germany (1%) | Europe (99%) | 2021-06-11 to 2021-08-20 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),Spike_T95I (99%),Spike_G142D (99%),NSP12_C730F (90%) |
| **583899** | 204/306 | USA (99%), Portugal (1%) | North America (99%) | 2021-06-11 to 2021-08-19 | **Active** | B.1.617.2 (94%) | GK (97%) | Delta | N_D63G (100%),NSP12_G671S (100%),Spike_V289I (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_E815D (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_D950N (100%),N_R203M (99%),N_G18V (99%) |
| **583960** | 39/150 | United Kingdom (95%), Norway (1%), Romania (1%), Hong Kong (1%), Sweden (1%) | Europe (99%) | 2021-06-11 to 2021-08-21 | **Active** | B.1.617.2 (62%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (99%),NS7a_T120I (99%),Spike_G142D (99%),NSP4_V3F (96%) |
| **584877** | 168/258 | USA (96%), Belgium (3%), France (1%), Mexico (0%), Japan (0%) | North America (96%) | 2021-05-16 to 2021-08-19 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | NSP4_D217N (100%),N_D63G (100%),N_R203M (100%),NS8_E19stop (100%),NS3_S26L (100%),NSP4_F375S (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NSP3_P822L (100%),Spike_E156G (100%),NSP15_K259R (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP2_P129L (100%),Spike_K77T (100%),NS7a_T120I (100%),Spike_D950N (99%),NSP6_H11Q (95%),Spike_G142D (71%),Spike_T547I (64%) |
| **585031** | 32/39 | United Kingdom (85%), USA (13%), Turkey (3%) | Europe (87%) | 2021-05-31 to 2021-08-22 | **Active** | B.1.617.2 (54%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (97%),NSP6_T77A (97%),Spike_G142D (95%),Spike_T95I (92%),NSP14_A394V (87%),NSP3_A1711V (79%) |
| **585126** | 98/170 | USA (84%), Denmark (14%), United Kingdom (2%), Germany (1%) | North America (84%) | 2021-05-31 to 2021-08-21 | **Active** | B.1.617.2 (98%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_E661D (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP6_A2V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (98%) |
| **585319** | 607/984 | USA (96%), Denmark (2%), Japan (0%), Netherlands (0%), Switzerland (0%) | North America (97%) | 2021-06-07 to 2021-08-20 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NS6_E46V (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_D63G (100%),Spike_P681R (100%),M_I82T (100%),Spike_L452R (100%),NS7b_T40I (100%),Spike_T95I (100%),NSP13_P77L (100%),NSP6_T77A (100%),Spike_D950N (99%),Spike_G142D (95%) |
| **585482** | 1294/2165 | USA (99%), Canada (0%), Japan (0%), Hong Kong (0%), Netherlands (0%) | North America (100%) | 2021-04-27 to 2021-08-21 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_G215C (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_E156G (100%),Spike_T478K (100%),NS8_L60F (100%),NSP14_A394V (100%),NSP13_H164Y (100%),N_D377Y (100%),N_D63G (100%),NSP12_F192V (100%),NS7a_V82A (100%),NSP3_P1469S (100%),N_R203M (100%),NSP3_A1736V (100%),NS7a_T120I (99%),NSP3_H1307Y (99%),Spike_L452R (99%),NSP6_T77A (99%),Spike_D950N (99%),Spike_G142D (92%) |
| **585688** | 27/60 | United Kingdom (100%) | Europe (100%) | 2021-06-13 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_D142G (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **585774** | 170/345 | United Kingdom (100%), Singapore (0%) | Europe (100%) | 2021-05-27 to 2021-08-23 | **Active** | AY.4 (61%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_Y248H (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),NSP5_A260V (98%) |
| **585999** | 227/334 | USA (100%), United Kingdom (0%) | North America (100%) | 2021-06-13 to 2021-08-22 | **Active** | AY.3 (99%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NS3_D210A (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_V70I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),NSP12_G671S (100%),NSP14_A394V (100%),NSP13_P77L (100%),NSP6_I162V (99%),NSP6_T77A (99%),NSP12_P323L (99%),Spike_D950N (97%),Spike_G142D (87%) |
| **586306** | 115/246 | United Kingdom (100%) | Europe (100%) | 2021-06-14 to 2021-08-23 | **Active** | B.1.617.2 (82%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (100%),N_L139F (98%) |
| **586313** | 68/113 | United Kingdom (98%), France (2%) | Europe (100%) | 2021-06-14 to 2021-08-18 | **Active** | AY.4 (58%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NSP3_K1291E (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_K235T (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_M72I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_A871S (100%),N_D63G (99%) |
| **586364** | 797/2153 | Denmark (55%), United Kingdom (43%), Sweden (1%), Germany (0%), Netherlands (0%) | Europe (100%) | 2021-05-01 to 2021-08-24 | **Active** | B.1.617.2 (58%) | GK (100%) | Delta | NS7a_A79D (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP9_Q11H (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (100%),NS7a_C58F (100%),Spike_T19R (100%),Spike_G142D (100%) |
| **586400** | 36/59 | United Kingdom (20%), Switzerland (15%), Japan (15%), USA (15%), France (12%) | Europe (63%) | 2021-04-20 to 2021-08-18 | **Active** | B.1.617.2 (54%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_G671S (98%),Spike_D950N (98%),NSP12_P323L (97%),Spike_T95I (95%),Spike_G142D (61%) |
| **586482** | 168/482 | Denmark (78%), Germany (10%), United Kingdom (3%), Netherlands (2%), Italy (1%) | Europe (99%) | 2021-01-07 to 2021-08-23 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),Spike_H66Y (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),Spike_D950N (100%),Spike_G142D (90%) |
| **586491** | 85/148 | USA (100%) | North America (100%) | 2021-05-28 to 2021-08-18 | **Active** | B.1.617.2 (86%) | GK (97%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS7a_L116F (99%),NS7a_V82A (99%),NSP14_S374F (99%),NS7a_T120I (99%) |
| **586511** | 62/113 | United Kingdom (100%) | Europe (100%) | 2021-06-14 to 2021-08-22 | **Active** | B.1.617.2 (61%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_T64I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_G142D (97%) |
| **586746** | 103/136 | France (51%), Italy (15%), Germany (12%), Switzerland (8%), Belgium (2%) | Europe (98%) | 2021-06-14 to 2021-08-20 | **Active** | AY.12 (77%) | GK (94%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_A318V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS7a_R118G (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NS7a_V82A (99%),NSP3_P1228L (99%),Spike_D950N (99%),NSP12_P323L (98%) |
| **587122** | 49/78 | USA (99%), United Kingdom (1%) | North America (99%) | 2021-06-11 to 2021-08-17 | **Active** | AY.3 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_I162V (99%),Spike_G142D (99%),NSP3_A1798V (99%) |
| **587415** | 521/878 | USA (98%), United Kingdom (1%), Canada (0%), Japan (0%), Sint Maarten (0%) | North America (99%) | 2021-06-14 to 2021-08-22 | **Active** | B.1.617.2 (90%) | GK (100%) | Delta | N_G215C (100%),N_A208S (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D63G (100%),N_R203M (100%),NSP3_P1228L (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP3_P1469S (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_P681R (100%),NSP3_A488S (100%),NS3_P104S (99%),NSP6_T77A (99%),Spike_D950N (98%),Spike_G142D (91%) |
| **587876** | 186/388 | United Kingdom (99%), Spain (1%), USA (0%) | Europe (100%) | 2021-06-15 to 2021-08-23 | **Active** | AY.4 (69%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),M_A81S (100%),NSP12_P323L (100%),N_P365L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (100%),Spike_D950N (100%),Spike_G142D (99%) |
| **587980** | 271/667 | United Kingdom (99%), Italy (0%), Hong Kong (0%), USA (0%), Denmark (0%) | Europe (100%) | 2021-06-07 to 2021-08-23 | **Active** | AY.4 (62%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_P1162S (100%),NS7a_V82A (100%),Spike_G142D (100%) |
| **587999** | 70/251 | United Kingdom (82%), Canada (14%), USA (4%), Japan (0%) | Europe (82%) | 2021-04-23 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_T710I (80%),NSP6_A2V (80%) |
| **588107** | 39/80 | United Kingdom (100%) | Europe (100%) | 2021-06-15 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_Q386H (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (98%),NSP2_A205V (95%) |
| **588396** | 331/802 | Netherlands (52%), Aruba (19%), Germany (6%), Denmark (6%), Italy (3%) | Europe (78%) | 2021-06-15 to 2021-08-18 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),NSP13_E319D (100%),N_R203M (100%),E_V58F (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP12_M463I (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (100%),N_D377Y (100%),Spike_T95I (100%),NSP3_A488S (100%),N_D63G (100%),Spike_D950N (98%),Spike_G142D (95%) |
| **589326** | 66/119 | United Kingdom (97%), Germany (3%) | Europe (100%) | 2021-06-16 to 2021-08-21 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS8_V33F (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP4_V233I (100%),NSP3_P1228L (100%),NS7a_R80I (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_S180F (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NSP6_L37F (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **589405** | 37/121 | United Kingdom (90%), Italy (7%), Switzerland (2%), Israel (1%) | Europe (99%) | 2021-06-16 to 2021-08-18 | **Active** | AY.4 (98%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (99%),Spike_T19R (99%),Spike_T478K (99%),NSP6_M143I (99%),Spike_D950N (97%),Spike_G142D (93%) |
| **589417** | 169/332 | United Kingdom (98%), Portugal (1%), Netherlands (1%), Japan (0%), Poland (0%) | Europe (100%) | 2021-06-16 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_D1127G (100%),NS3_S26L (100%),Spike_P251L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP3_E906D (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NSP1_E87D (100%),N_G215C (100%),Spike_R158del (100%),Spike_E156G (100%),Spike_L452R (100%),NS8_A65S (99%),Spike_G142D (99%) |
| **589781** | 546/919 | Denmark (24%), France (24%), Netherlands (19%), Spain (9%), Switzerland (6%) | Europe (97%) | 2021-06-01 to 2021-08-24 | **Active** | B.1.617.2 (84%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (100%),NSP3_P1228L (100%),N_D377Y (100%),NS7a_T120I (100%),Spike_T95I (99%),NSP2_V485I (98%),Spike_D950N (97%),Spike_T719I (94%),Spike_G142D (84%) |
| **590026** | 293/402 | USA (52%), Mexico (46%), United Kingdom (1%), Germany (0%), Canada (0%) | North America (98%) | 2021-06-04 to 2021-08-21 | **Active** | B.1.617.2 (74%) | GK (98%) | Delta | N_G215C (100%),NSP12_G671S (100%),NSP2_A419V (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),N_R203M (100%),NSP13_P77L (100%),NSP6_T77A (100%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_T95I (99%),Spike_D950N (86%),Spike_G142D (73%) |
| **590111** | 153/230 | USA (100%) | North America (100%) | 2021-05-10 to 2021-08-18 | **Active** | B.1.617.2 (95%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS6_Q56R (100%),NSP6_T181I (100%),NSP3_T724I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP14_Y494H (100%),Spike_F157del (100%),Spike_T19R (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS7a_V82A (100%),NS7a_T120I (99%),Spike_D950N (97%),NSP3_N51K (86%),Spike_A262S (75%) |
| **590118** | 187/326 | USA (98%), North Macedonia (1%), Sint Maarten (0%), Japan (0%), Denmark (0%) | North America (98%) | 2021-06-16 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS7b_E33V (100%),NSP15_D132Y (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP14_S374F (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP16_M20I (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP2_S358A (100%),Spike_D950N (100%),NSP2_P129L (99%),NSP3_P822L (99%),Spike_G142D (99%) |
| **590148** | 79/269 | Germany (40%), Switzerland (13%), Netherlands (9%), France (8%), Spain (7%) | Europe (100%) | 2021-06-16 to 2021-08-19 | **Active** | AY.12 (53%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_V485I (100%),M_I82T (100%),Spike_T719I (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (100%),Spike_F157del (100%),N_D377Y (100%),Spike_E156G (100%),NSP12_D824Y (99%),Spike_D950N (99%),Spike_T95I (98%),N_D63G (97%) |
| **590369** | 74/250 | Turkey (90%), Denmark (6%), Germany (4%) | Europe (100%) | 2021-05-25 to 2021-08-19 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP4_F375S (100%),Spike_P681R (100%),NS8_R52I (100%),NS7a_T120I (100%),NSP1_F143del (100%),N_D377Y (100%),M_I82T (100%),NSP1_K141del (100%),NSP1_S142del (100%),NSP15_K259R (100%),Spike_D614G (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),NSP6_H11Q (100%),NS3_S26L (99%),NS7a_V82A (99%),Spike_T19R (99%),NSP3_P822L (99%),Spike_T478K (99%),NSP2_P129L (98%),Spike_D950N (98%),Spike_K77T (97%),NSP12_P323L (97%),Spike_L452R (97%),NSP4_D217N (94%),Spike_G142D (60%) |
| **590945** | 71/163 | United Kingdom (97%), Netherlands (1%), Belgium (1%), Denmark (1%) | Europe (100%) | 2021-06-17 to 2021-08-20 | **Active** | B.1.617.2 (87%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP14_A394V (99%),Spike_P681R (98%) |
| **591082** | 57/124 | United Kingdom (91%), Turkey (6%), USA (2%), France (1%), Indonesia (1%) | Europe (98%) | 2021-05-17 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_A90S (100%),NSP3_A1711V (92%),Spike_T95I (92%),NSP1_F143del (60%),NSP1_K141del (60%),NSP1_S142del (60%) |
| **591569** | 49/119 | Turkey (37%), Denmark (18%), Sweden (16%), Germany (14%), Norway (4%) | Europe (98%) | 2021-06-17 to 2021-08-24 | **Active** | B.1.617.2 (76%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),Spike_T250I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),Spike_T29A (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_D950N (99%),N_D63G (98%),NSP13_P77L (96%),NSP4_V167L (96%),Spike_L452R (96%),NSP15_D282Y (83%),Spike_G142D (76%) |
| **591699** | 22/27 | USA (48%), United Kingdom (30%), France (15%), India (4%), Denmark (4%) | North America (48%) | 2021-05-06 to 2021-08-21 | **Active** | B.1.617.2 (85%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_L5F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **591704** | 63/122 | USA (98%), Japan (1%), United Kingdom (1%) | North America (98%) | 2021-06-11 to 2021-08-17 | **Active** | AY.3 (89%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_I162V (99%),NS7a_T120I (99%),NSP3_P1469S (98%),Spike_D950N (95%),Spike_G142D (84%),NSP3_W1509C (80%),NSP8_T141M (80%) |
| **592451** | 263/1017 | United Kingdom (91%), Turkey (3%), Germany (1%), Denmark (1%), Switzerland (1%) | Europe (99%) | 2021-04-23 to 2021-08-23 | **Active** | AY.4 (60%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),Spike_G142D (98%),NSP6_T77A (97%),Spike_D950N (96%),NSP3_A1711V (95%) |
| **592540** | 42/82 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-06-18 to 2021-08-18 | **Active** | AY.4 (74%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP16_S276I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP14_S418I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_S939F (99%),Spike_L18F (99%) |
| **592620** | 104/230 | United Kingdom (100%), Spain (0%) | Europe (100%) | 2021-06-18 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7a_V24F (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_G142D (100%),Spike_L452R (100%) |
| **592677** | 31/85 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-18 to 2021-08-23 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),NS3_G49V (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),NSP2_D511N (96%),NSP13_E365D (96%) |
| **592699** | 24/54 | United Kingdom (98%), Denmark (2%) | Europe (100%) | 2021-06-18 to 2021-08-21 | **Active** | AY.4 (98%) | GK (96%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP13_A110S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_T246I (98%) |
| **592811** | 56/93 | United Kingdom (100%) | Europe (100%) | 2021-05-12 to 2021-08-22 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_T634I (78%) |
| **593128** | 126/170 | Mexico (52%), USA (42%), France (4%), Japan (2%), Belgium (1%) | North America (94%) | 2021-05-30 to 2021-08-19 | **Active** | B.1.617.2 (85%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_A419V (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_T95I (99%),Spike_K97E (99%),Spike_D950N (91%),Spike_G142D (84%) |
| **593292** | 111/175 | USA (69%), Mexico (21%), Chile (3%), Guatemala (2%), Finland (2%) | North America (93%) | 2020-09-22 to 2021-08-19 | **Active** | B.1.617.2 (95%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP13_A296S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_T223I (99%),Spike_T95I (99%),NS7b_T40I (98%),N_D63G (98%),Spike_D950N (91%),Spike_G142D (63%) |
| **593457** | 78/117 | USA (99%), Aruba (1%) | North America (99%) | 2021-06-17 to 2021-08-17 | **Active** | AY.2 (99%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_K417N (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP13_T367I (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NSP3_A328T (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_T478K (99%),Spike_V70F (98%),NS3_T268M (98%),Spike_D950N (97%),Spike_G142D (79%) |
| **593575** | 2253/4630 | Denmark (20%), France (17%), Netherlands (14%), United Kingdom (10%), Italy (9%) | Europe (99%) | 2021-05-07 to 2021-08-27 | **Active** | B.1.617.2 (82%) | GK (99%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),NSP3_P1228L (100%),NS7a_T120I (100%),NSP13_P77L (100%),N_G215C (100%),Spike_P681R (100%),NSP4_T492I (100%),N_D377Y (100%),M_I82T (100%),N_Q9L (100%),NS7b_T40I (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),N_R203M (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_T19R (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_L452R (100%),NSP14_A394V (100%),NSP3_P1469S (100%),Spike_F157del (100%),Spike_R158del (100%),Spike_E156G (100%),NSP12_L838I (100%),N_D63G (99%),Spike_D950N (98%),Spike_G142D (82%) |
| **593857** | 73/228 | Australia (100%) | Oceania (100%) | 2021-06-19 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (82%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NSP16_Q238H (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),NSP3_A488S (100%),Spike_D950N (100%),NS7a_V71I (100%),N_D63G (99%),NS7a_V82A (99%) |
| **593879** | 39/105 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-19 to 2021-08-23 | **Active** | B.1.617.2 (63%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NS3_P240L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),Spike_G142D (99%),Spike_T95I (98%) |
| **593922** | 58/97 | United Kingdom (99%), Bangladesh (1%) | Europe (99%) | 2021-06-18 to 2021-08-22 | **Active** | B.1.617.2 (78%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NSP2_F356L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),N_A35T (97%) |
| **593948** | 52/150 | United Kingdom (98%), South Korea (1%), Denmark (1%), Belgium (1%) | Europe (99%) | 2021-06-19 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_A1461V (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **593950** | 43/79 | United Kingdom (95%), USA (3%), France (3%) | Europe (97%) | 2021-06-19 to 2021-08-22 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS3_T175I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP3_A1711V (95%) |
| **593970** | 50/76 | United Kingdom (96%), India (1%), Denmark (1%), Sweden (1%) | Europe (99%) | 2021-05-23 to 2021-08-22 | **Active** | AY.4 (84%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),NS3_G172C (96%),NSP4_V3F (63%) |
| **594127** | 77/171 | USA (99%), Aruba (1%) | North America (99%) | 2021-06-15 to 2021-08-21 | **Active** | AY.3 (100%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),NSP9_L42F (94%),NSP15_A80V (94%),Spike_G142D (88%) |
| **594152** | 37/96 | Sweden (49%), United Kingdom (30%), South Africa (4%), Portugal (3%), Denmark (3%) | Europe (93%) | 2021-06-14 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (95%),NSP12_M124I (60%) |
| **594631** | 46/110 | United Kingdom (99%), Germany (1%) | Europe (100%) | 2021-06-20 to 2021-08-22 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_P279S (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NS7a_P45S (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **594632** | 37/113 | United Kingdom (95%), Denmark (2%), Norway (1%), Switzerland (1%), Italy (1%) | Europe (100%) | 2021-06-20 to 2021-08-20 | **Active** | B.1.617.2 (57%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_T19R (99%),NS7a_T120I (99%),Spike_G142D (90%) |
| **594722** | 65/121 | United Kingdom (100%) | Europe (100%) | 2021-06-20 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A890D (88%) |
| **594788** | 42/92 | United Kingdom (100%) | Europe (100%) | 2021-06-20 to 2021-08-22 | **Active** | AY.4 (60%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_D138H (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP5_A129V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_A522S (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (99%) |
| **594818** | 36/70 | United Kingdom (100%) | Europe (100%) | 2021-06-20 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS8_Q18stop (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),NS8_W45L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **594881** | 170/370 | Australia (100%) | Oceania (100%) | 2021-06-16 to 2021-08-19 | **Active** | AY.12 (95%) | GK (89%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NSP16_Q238H (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_V71I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),NSP14_A394V (100%),Spike_D950N (98%) |
| **594933** | 215/710 | Denmark (28%), Sweden (25%), France (11%), Netherlands (7%), Norway (6%) | Europe (97%) | 2021-06-20 to 2021-08-24 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_D63G (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_R203M (100%),NSP12_G671S (100%),NSP14_T16I (100%),Spike_T19R (100%),NSP12_R197Q (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_G142D (94%) |
| **595185** | 138/177 | USA (100%) | North America (100%) | 2021-05-14 to 2021-08-19 | **Active** | AY.4 (97%) | GK (99%) | Delta | N_D63G (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (99%),N_R203M (99%),NSP3_P1228L (99%),Spike_D950N (99%),NSP12_P323L (99%) |
| **595403** | 301/480 | Germany (21%), Switzerland (18%), France (13%), Italy (12%), Belgium (10%) | Europe (99%) | 2021-06-15 to 2021-08-19 | **Active** | AY.9 (96%) | GK (94%) | Delta | NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS7a_V82A (100%),N_R203M (100%),Spike_D950N (99%),N_D63G (97%),NSP14_T16I (90%),NSP12_R197Q (90%) |
| **595468** | 29/59 | United Kingdom (100%) | Europe (100%) | 2021-06-21 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (97%) | Delta | NS7a_A79D (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_Y265C (100%),NSP3_P1228L (100%),NSP9_Q11H (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_C58F (93%) |
| **595503** | 69/128 | United Kingdom (99%), Denmark (1%) | Europe (100%) | 2021-06-21 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_I1114T (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_D138H (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **595533** | 24/49 | United Kingdom (82%), USA (18%) | Europe (82%) | 2021-06-07 to 2021-08-20 | **Active** | B.1.617.2 (96%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (98%),Spike_G142D (98%),Spike_R158del (98%),Spike_F157del (98%),Spike_E156G (98%),Spike_D936H (94%),NSP3_A1711V (82%) |
| **595630** | 46/95 | United Kingdom (99%), Norway (1%) | Europe (100%) | 2021-06-21 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),M_S212I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **595636** | 35/52 | United Kingdom (98%), Portugal (2%) | Europe (100%) | 2021-05-22 to 2021-08-22 | **Active** | AY.4 (71%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%),NSP3_V965F (96%) |
| **596278** | 40/121 | USA (100%) | North America (100%) | 2021-06-21 to 2021-08-19 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (98%),Spike_D950N (98%),Spike_G142D (92%) |
| **596352** | 51/65 | South Africa (28%), Sweden (14%), Netherlands (11%), USA (9%), United Kingdom (8%) | Europe (54%) | 2021-05-21 to 2021-08-19 | **Active** | B.1.617.2 (63%) | GK (92%) | Delta | N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_T250S (100%),N_D63G (98%),NSP12_G774C (98%),Spike_D950N (91%),Spike_G142D (63%) |
| **596525** | 27/55 | United Kingdom (98%), Norway (2%) | Europe (100%) | 2021-06-21 to 2021-08-18 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_D1318N (100%),NS7a_V82A (100%),NSP3_S1424F (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),M_A63V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **596571** | 299/415 | USA (83%), Mexico (10%), United Kingdom (3%), Brazil (3%), Malaysia (0%) | North America (93%) | 2021-04-16 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),N_D377Y (100%),Spike_E156G (100%),NS7a_T120I (100%),NSP3_P1469S (100%),Spike_T95I (99%),Spike_D950N (97%),NSP13_A296S (92%),Spike_G142D (90%) |
| **596922** | 57/107 | Slovenia (80%), Germany (12%), Switzerland (3%), Italy (2%), France (1%) | Europe (100%) | 2021-05-20 to 2021-08-19 | **Active** | AY.4 (99%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NSP12_K91R (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_A119V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G1167V (99%),Spike_D950N (95%) |
| **596944** | 56/86 | USA (99%), Netherlands (1%) | North America (99%) | 2021-06-20 to 2021-08-19 | **Active** | AY.3 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP3_A1798V (100%),Spike_D950N (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7b_T40I (99%),NSP3_P1469S (90%),Spike_G142D (83%) |
| **596974** | 25/34 | Malaysia (76%), Singapore (21%), Japan (3%) | Asia (100%) | 2021-06-04 to 2021-08-19 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NSP16_P215L (100%),Spike_T478K (100%),NS8_C20R (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (82%) |
| **597133** | 77/133 | USA (58%), Mexico (24%), United Kingdom (9%), Brazil (2%), Spain (2%) | North America (83%) | 2021-06-04 to 2021-08-18 | **Active** | B.1.621 (100%) | GH (99%) | Mu | Spike_E484K (100%),NSP3_T720I (100%),NS3_V256I (100%),Spike_N501Y (100%),Spike_P681H (100%),NSP14_V125F (100%),NS8_P38S (100%),NSP4_T492I (100%),NSP13_P419S (100%),NSP6_Q160R (100%),NSP13_P172H (100%),Spike_R346K (100%),N_T205I (100%),NS3_Q57H (100%),NS8_T11K (100%),NSP3_T237A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_T95I (99%),NSP2_A294S (94%),Spike_D950N (93%),Spike_K417N (93%),NS8_S67F (91%),Spike_Y145S (89%),Spike_ins145N (89%),Spike_Y144T (89%) |
| **597317** | 38/55 | United Kingdom (95%), France (4%), Japan (2%) | Europe (98%) | 2021-06-17 to 2021-08-23 | **Active** | AY.4 (71%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_A688V (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (95%),NSP3_E1091A (64%) |
| **597347** | 48/112 | United Kingdom (99%), Hong Kong (1%) | Europe (99%) | 2021-06-08 to 2021-08-22 | **Active** | B.1.617.2 (56%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **597359** | 64/257 | United Kingdom (98%), Finland (1%), Portugal (0%), Belgium (0%), USA (0%) | Europe (100%) | 2021-06-05 to 2021-08-23 | **Active** | B.1.617.2 (53%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_G142D (98%) |
| **597459** | 75/176 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-05-22 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS3_A99V (99%),Spike_P681R (99%),NS7a_V82A (99%),NSP2_D511N (99%),Spike_I1115V (99%),NSP13_E365D (99%),Spike_G142D (98%) |
| **598378** | 179/280 | USA (92%), Mexico (4%), United Kingdom (2%), Switzerland (1%), France (0%) | North America (96%) | 2021-05-15 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS6_K48N (100%),NSP6_T181I (100%),Spike_P681R (100%),NS7a_T120I (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_D63G (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),Spike_A222V (100%),N_D377Y (100%),Spike_E156G (100%),Spike_N1074S (99%),NSP3_P822L (99%),Spike_D950N (99%),Spike_V1264L (98%),Spike_G142D (92%) |
| **598549** | 82/145 | USA (86%), Sweden (14%) | North America (86%) | 2021-06-22 to 2021-08-20 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_Q675H (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_A323S (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP4_V167L (100%),Spike_T95I (99%),NSP4_A92V (99%),Spike_D950N (97%),Spike_G142D (92%) |
| **598593** | 72/113 | USA (99%), Sint Maarten (1%) | North America (100%) | 2021-06-22 to 2021-08-17 | **Active** | AY.3 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_D210A (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_V70I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_M177I (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (92%) |
| **598596** | 1197/2123 | USA (53%), United Kingdom (27%), India (9%), Canada (2%), Turkey (1%) | North America (55%) | 2021-03-19 to 2021-08-19 | **Active** | B.1.617.2 (94%) | GK (98%) | Delta | Spike_P681R (100%),NSP13_P77L (100%),Spike_D614G (100%),N_R203M (100%),NSP12_G671S (100%),M_I82T (100%),Spike_L452R (100%),Spike_T19R (100%),N_D377Y (100%),Spike_T478K (100%),NS7a_V82A (100%),NSP3_P822L (100%),NS7a_T120I (100%),NSP12_P323L (100%),NSP4_A446V (100%),NS7a_L116F (100%),NSP6_V149A (100%),Spike_R158del (100%),NSP2_P129L (99%),Spike_F157del (99%),Spike_E156G (99%),NS3_S26L (99%),NSP14_P46L (99%),N_D63G (99%),Spike_D950N (98%),Spike_G142D (87%) |
| **598900** | 283/834 | Denmark (28%), France (18%), Netherlands (10%), Spain (8%), United Kingdom (6%) | Europe (99%) | 2021-06-22 to 2021-08-27 | **Active** | B.1.617.2 (82%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_T19R (100%),N_Q9L (100%),Spike_T478K (100%),NSP4_V167L (100%),NS7a_V82A (100%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),N_D63G (99%),Spike_D950N (97%),Spike_G142D (82%) |
| **599067** | 1236/1956 | USA (96%), Canada (2%), Sint Maarten (1%), Aruba (0%), Mexico (0%) | North America (99%) | 2021-04-29 to 2021-08-21 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_T478K (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),N_G215C (100%),NSP3_A488S (100%),NSP4_T492I (100%),NS7a_T120I (100%),Spike_L452R (100%),NS3_S26L (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP4_V167L (100%),NSP3_P1469S (100%),N_D63G (100%),NSP3_P1228L (100%),NSP14_A394V (100%),NSP6_T77A (99%),Spike_D950N (98%),Spike_G142D (93%),NSP6_C197F (86%),NSP3_D410G (82%) |
| **599321** | 280/384 | Indonesia (58%), Japan (9%), USA (7%), France (5%), Germany (4%) | Asia (76%) | 2021-03-15 to 2021-08-17 | **Active** | B.1.617.2 (99%) | GK (95%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_P104S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_A222V (100%),NSP3_P822L (100%),NSP12_P323L (100%),Spike_D950N (95%),Spike_G142D (90%) |
| **599437** | 128/225 | USA (99%), Japan (1%) | North America (99%) | 2021-06-22 to 2021-08-20 | **Active** | B.1.617.2 (84%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS8_C25Y (100%),NSP6_T77A (100%),Spike_D950N (98%),Spike_G142D (84%) |
| **599462** | 49/72 | USA (99%), Portugal (1%) | North America (99%) | 2021-06-22 to 2021-08-20 | **Active** | B.1.617.2 (90%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP12_F192V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP5_A260V (100%),NSP12_P323L (100%),NS8_Q72H (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (89%) |
| **599713** | 149/201 | Mexico (37%), USA (29%), United Kingdom (14%), Turkey (7%), Japan (3%) | North America (67%) | 2021-04-14 to 2021-08-19 | **Active** | B.1.617.2 (81%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),Spike_L452R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NSP4_V167L (96%),Spike_D950N (90%),Spike_G142D (86%),NSP2_A419V (60%) |
| **599806** | 25/37 | United Kingdom (100%) | Europe (100%) | 2021-06-23 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_V1133I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_K1073M (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **599855** | 183/392 | United Kingdom (100%), USA (0%) | Europe (100%) | 2021-06-22 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),N_P383S (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_D511N (99%),NSP13_E365D (99%),NSP15_A217S (98%) |
| **600091** | 69/137 | United Kingdom (76%), USA (12%), Turkey (6%), Germany (4%), Denmark (1%) | Europe (88%) | 2021-06-20 to 2021-08-22 | **Active** | B.1.617.2 (94%) | GK (99%) | Delta | N_G215C (99%),Spike_T95I (99%),N_D63G (99%),N_R203M (99%),NSP12_G671S (99%),NS3_S26L (99%),Spike_P681R (99%),Spike_R158del (99%),NSP3_P1228L (99%),NS7a_V82A (99%),NSP3_A488S (99%),Spike_F157del (99%),NSP4_T492I (99%),NSP14_A394V (99%),Spike_T19R (99%),NS7a_T120I (99%),N_D377Y (99%),M_I82T (99%),NS7b_T40I (99%),NSP13_P77L (99%),Spike_E156G (99%),NSP3_P1469S (99%),Spike_T478K (99%),NS7a_P45L (99%),NSP6_T77A (99%),NSP12_P323L (99%),Spike_D614G (99%),NSP4_V167L (99%),Spike_L452R (99%),Spike_D950N (99%),Spike_G142D (94%),NSP3_A1711V (79%) |
| **600106** | 40/93 | United Kingdom (100%) | Europe (100%) | 2021-06-23 to 2021-08-22 | **Active** | AY.4 (99%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_T223I (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NS3_G100C (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_S255F (100%),NS3_D155H (91%) |
| **600143** | 42/84 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-06-04 to 2021-08-18 | **Active** | B.1.617.2 (95%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_Q414H (99%) |
| **600392** | 141/265 | USA (100%), Belgium (0%) | North America (100%) | 2021-04-25 to 2021-08-21 | **Active** | B.1.617.2 (91%) | GK (98%) | Delta | N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (100%),N_D63G (100%),Spike_R158del (100%),Spike_F157del (100%),N_D377Y (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP6_T77A (100%),Spike_A879S (100%),NSP4_V167L (100%),NS3_S26L (99%),NS7a_A13T (99%),NSP2_K81N (99%),NSP16_D269E (99%),Spike_G142D (92%) |
| **601710** | 50/115 | United Kingdom (95%), Canada (4%), Portugal (1%) | Europe (96%) | 2021-06-15 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_D138H (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP5_A129V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_S327L (85%) |
| **601716** | 47/57 | United Kingdom (91%), USA (4%), Netherlands (2%), Japan (2%), Sweden (2%) | Europe (95%) | 2021-06-09 to 2021-08-21 | **Active** | B.1.617.2 (79%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),Spike_G1124V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (95%) |
| **601858** | 42/81 | United Kingdom (100%) | Europe (100%) | 2021-06-24 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),NS8_V100L (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **601870** | 52/96 | United Kingdom (96%), Denmark (2%), Sweden (1%), USA (1%) | Europe (99%) | 2021-06-07 to 2021-08-18 | **Active** | AY.4 (53%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q14H (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),NSP16_S276I (91%) |
| **601942** | 188/370 | United Kingdom (97%), France (2%), Gibraltar (0%), Netherlands (0%) | Europe (100%) | 2021-06-18 to 2021-08-23 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),NS3_S26L (100%),Spike_D950N (100%),Spike_G142D (99%) |
| **602224** | 58/65 | USA (49%), Brazil (25%), Finland (9%), Switzerland (6%), Germany (5%) | North America (49%) | 2021-05-20 to 2021-08-18 | **Active** | AY.4 (83%) | GK (97%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),Spike_L452R (100%),N_D63G (98%),NSP12_P323L (98%),NSP4_V167L (98%),NSP14_A394V (94%) |
| **602271** | 39/84 | United Kingdom (100%) | Europe (100%) | 2021-06-22 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),Spike_V83F (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **602309** | 37/163 | United Kingdom (98%), Netherlands (1%), Hong Kong (1%), Belgium (1%) | Europe (99%) | 2021-06-24 to 2021-08-22 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_L323F (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **602444** | 39/97 | France (59%), Poland (8%), Netherlands (7%), Sweden (7%), Denmark (4%) | Europe (98%) | 2021-06-24 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_L41F (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P251L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_A65S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (99%),NSP3_E906D (98%) |
| **602772** | 47/98 | Italy (80%), Switzerland (10%), Germany (4%), Denmark (2%), Netherlands (1%) | Europe (100%) | 2021-06-24 to 2021-08-19 | **Active** | AY.12 (67%) | GK (93%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q677H (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP13_V479I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP14_A394V (99%),NSP4_V167L (99%),N_D63G (96%) |
| **603009** | 167/257 | USA (98%), Aruba (1%), Costa Rica (0%), Netherlands (0%) | North America (99%) | 2021-06-21 to 2021-08-18 | **Active** | B.1.617.2 (90%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),M_T30I (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (100%),Spike_F157del (100%),NS7a_T120I (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NS3_E239Q (100%),Spike_D950N (99%),Spike_S112L (99%),Spike_G142D (90%) |
| **603196** | 221/498 | Italy (28%), Slovenia (22%), France (15%), Germany (11%), Belgium (9%) | Europe (99%) | 2021-06-25 to 2021-08-19 | **Active** | B.1.617.2 (70%) | GK (98%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_P251L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7b_T40I (100%),NSP3_P1469S (100%),NSP6_T77A (100%),NS7a_V82A (100%),M_I82T (100%),NSP1_E87D (100%),N_R203M (99%),NSP3_E906D (99%),NSP4_V167L (99%),NS8_A65S (99%),Spike_D950N (99%),N_D63G (97%) |
| **603414** | 64/111 | USA (97%), Canada (1%), United Kingdom (1%), Denmark (1%) | North America (98%) | 2021-06-24 to 2021-08-18 | **Active** | B.1.617.2 (85%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS3_D238Y (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),NSP13_M474I (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),NSP6_T77A (99%),NSP16_M270I (90%),Spike_G142D (86%) |
| **604060** | 92/148 | USA (97%), United Kingdom (2%), Japan (1%) | North America (97%) | 2021-06-25 to 2021-08-18 | **Active** | B.1.617.2 (97%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_T95I (99%),Spike_G142D (98%),NSP14_A323S (98%),NSP4_A92V (98%) |
| **604559** | 36/73 | United Kingdom (100%) | Europe (100%) | 2021-06-26 to 2021-08-21 | **Active** | B.1.617.2 (79%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_A50S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **604568** | 143/284 | United Kingdom (98%), USA (1%), Indonesia (0%), Mexico (0%), Germany (0%) | Europe (98%) | 2021-06-18 to 2021-08-22 | **Active** | AY.4 (97%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP4_T492I (100%),Spike_D950N (99%),Spike_T95I (98%),Spike_G142D (98%),NSP3_A1711V (98%),NSP2_E172D (83%) |
| **604577** | 75/140 | United Kingdom (43%), Italy (20%), Denmark (9%), Germany (9%), Norway (7%) | Europe (100%) | 2021-06-15 to 2021-08-17 | **Active** | B.1.617.2 (61%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_D1259Y (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_D950N (97%),NSP12_D481A (88%),Spike_G142D (66%) |
| **604673** | 73/143 | United Kingdom (99%), Norway (1%) | Europe (100%) | 2021-06-26 to 2021-08-22 | **Active** | AY.4 (100%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NS3_D155Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_D411N (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS8_V5A (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_D879Y (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_S23P (96%) |
| **604736** | 100/208 | United Kingdom (100%), Denmark (0%) | Europe (100%) | 2021-05-30 to 2021-08-23 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%),NS7a_T120I (100%) |
| **604758** | 65/121 | United Kingdom (98%), Denmark (1%), Italy (1%) | Europe (100%) | 2021-06-26 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_P45L (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (99%),NSP12_P21S (99%),NSP3_A579V (99%) |
| **604761** | 50/113 | United Kingdom (100%) | Europe (100%) | 2021-06-26 to 2021-08-22 | **Active** | B.1.617.2 (56%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_E554G (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A488S (97%) |
| **605255** | 247/379 | Germany (34%), Turkey (26%), Switzerland (7%), France (7%), Belgium (4%) | Europe (93%) | 2021-05-28 to 2021-08-18 | **Active** | AY.12 (71%) | GK (96%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_D950N (100%),NS7a_P45L (100%),N_D63G (99%),NSP4_T492I (99%),NSP3_P1469S (99%),NSP6_T77A (98%),N_G215C (98%),NSP3_P1228L (97%),NSP2_K81N (97%),NSP3_A488S (96%),NSP4_V167L (77%) |
| **605719** | 44/84 | United Kingdom (95%), Sweden (1%), Denmark (1%), Switzerland (1%), Poland (1%) | Europe (100%) | 2021-06-27 to 2021-08-19 | **Active** | AY.7 (100%) | GK (100%) | Delta | NS7a_A79D (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7a_C58F (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP9_Q11H (99%),Spike_D950N (99%),Spike_G142D (96%) |
| **605755** | 48/102 | United Kingdom (100%) | Europe (100%) | 2021-06-27 to 2021-08-22 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_F855S (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_P168S (91%) |
| **605787** | 345/659 | Italy (36%), Switzerland (25%), Germany (12%), Spain (4%), France (4%) | Europe (95%) | 2021-06-27 to 2021-08-19 | **Active** | AY.12 (79%) | GK (95%) | Delta | NSP12_G671S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_F157del (100%),NSP4_T492I (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_E156G (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_R203M (100%),NS3_S26L (100%),NSP3_A488S (100%),NS7b_T40I (100%),NSP3_P1469S (100%),N_G215C (100%),NS3_D27Y (100%),NSP4_V167L (99%),Spike_T95I (99%),NSP13_P77L (98%),N_D63G (98%),NSP14_A394V (98%),Spike_T19R (97%),Spike_T478K (97%),NS7a_V82A (96%),NSP12_Q822H (96%),NSP14_R289H (94%),Spike_D950N (93%) |
| **606149** | 39/108 | United Kingdom (99%), Belgium (1%) | Europe (100%) | 2021-06-27 to 2021-08-22 | **Active** | AY.4 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NS3_D155Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_D411N (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS8_V5A (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_D879Y (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP12_P323L (99%) |
| **606241** | 281/469 | USA (99%), Australia (0%), Israel (0%), United Kingdom (0%), Japan (0%) | North America (99%) | 2021-06-24 to 2021-08-20 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_S112L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_G671S (100%),NSP14_A394V (100%),NSP6_T77A (100%),NS3_E239Q (100%),NS7a_T120I (100%),NSP3_P1469S (99%),N_D63G (99%),Spike_D950N (98%),Spike_G142D (91%) |
| **606242** | 74/152 | Canada (50%), USA (48%), France (1%), Mexico (1%) | North America (99%) | 2021-04-28 to 2021-08-19 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_K373N (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (93%) |
| **606271** | 385/633 | USA (99%), Germany (0%), Japan (0%), United Kingdom (0%) | North America (99%) | 2021-06-21 to 2021-08-22 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP9_R39K (100%),Spike_D950N (98%),NSP3_P1469S (96%),Spike_G142D (92%) |
| **606548** | 31/45 | Spain (44%), Germany (16%), Netherlands (11%), United Kingdom (11%), Switzerland (7%) | Europe (100%) | 2021-06-22 to 2021-08-19 | **Active** | AY.12 (91%) | GK (96%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS7a_V82A (100%),NSP4_T492I (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NS6_E59stop (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (98%),NSP3_P1228L (98%),NSP2_S263F (98%),NSP3_A488S (98%),NSP14_A394V (98%),Spike_T19R (98%),Spike_D950N (98%),NSP3_P1469S (98%),NSP16_P215T (93%),Spike_R158del (89%),Spike_F157del (89%),Spike_E156G (89%) |
| **606746** | 101/225 | United Kingdom (88%), Denmark (4%), Netherlands (4%), Germany (2%), Switzerland (0%) | Europe (100%) | 2021-06-24 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_D142G (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **606790** | 73/122 | United Kingdom (98%), Denmark (1%), Sint Maarten (1%) | Europe (99%) | 2021-06-20 to 2021-08-21 | **Active** | B.1.617.2 (69%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP3_A1711V (99%),NS7a_V82A (99%),Spike_L18F (99%),NSP4_V167L (99%) |
| **606801** | 39/80 | United Kingdom (98%), USA (2%) | Europe (98%) | 2021-06-28 to 2021-08-22 | **Active** | AY.4 (79%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NS3_P42L (96%),NSP3_A1711V (82%) |
| **607146** | 96/217 | Netherlands (49%), Denmark (21%), Belgium (8%), Germany (6%), France (3%) | Europe (99%) | 2021-06-28 to 2021-08-20 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (100%),Spike_G142D (94%) |
| **607434** | 27/75 | United Kingdom (100%) | Europe (100%) | 2021-06-27 to 2021-08-23 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (96%),NSP13_H164Y (95%),NSP6_L37F (95%) |
| **607612** | 50/71 | USA (100%) | North America (100%) | 2021-06-14 to 2021-08-17 | **Active** | AY.3 (100%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T11I (100%),Spike_D950N (99%),Spike_G142D (92%) |
| **607628** | 25/46 | USA (100%) | North America (100%) | 2021-06-28 to 2021-08-17 | **Active** | B.1.617.2 (85%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP9_R39K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (98%),Spike_F157del (98%),Spike_D950N (98%),Spike_E156G (98%),Spike_G142D (83%) |
| **607664** | 92/133 | USA (99%), United Kingdom (1%) | North America (99%) | 2021-06-11 to 2021-08-20 | **Active** | B.1.617.2 (79%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NS8_Q72H (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),M_I82T (99%),Spike_D950N (99%),NS7a_T120I (98%),NSP12_F192V (98%),Spike_G142D (82%) |
| **607739** | 54/88 | USA (100%) | North America (100%) | 2021-06-27 to 2021-08-20 | **Active** | B.1.617.2 (94%) | GK (95%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP9_R39K (100%),NSP12_P323L (100%),Spike_D614G (100%),NS3_E181V (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NSP14_A394V (99%),Spike_D950N (99%),NSP3_P1469S (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),Spike_G142D (95%) |
| **607964** | 212/534 | Denmark (47%), Netherlands (12%), Switzerland (9%), Germany (7%), Italy (7%) | Europe (97%) | 2021-06-28 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS3_D27Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),N_R203M (100%),NS7a_V82A (100%),NSP14_A394V (100%),NSP14_R289H (100%),NSP3_P1469S (100%),Spike_D950N (99%),Spike_G142D (99%),NSP12_Q822H (91%) |
| **607998** | 472/948 | Italy (15%), France (15%), Denmark (15%), United Kingdom (14%), Sweden (11%) | Europe (98%) | 2021-06-28 to 2021-08-27 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (100%),NSP12_P323L (100%),N_D377Y (100%),NSP6_T77A (100%),N_D63G (100%),Spike_D950N (100%),Spike_G142D (99%) |
| **608158** | 238/397 | USA (34%), United Kingdom (19%), Italy (16%), Germany (7%), Denmark (6%) | Europe (58%) | 2021-03-23 to 2021-08-23 | **Active** | B.1.617.2 (86%) | GK (96%) | Delta | N_R203M (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NS3_D238Y (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NSP12_G671S (99%),Spike_P681R (99%),NS7a_V82A (99%),NSP14_A394V (99%),Spike_T19R (99%),N_G215C (99%),N_D63G (99%),NSP3_P1228L (99%),NSP2_K81N (99%),NSP4_T492I (99%),Spike_L452R (99%),Spike_D950N (97%),Spike_G142D (87%) |
| **608387** | 77/146 | USA (100%) | North America (100%) | 2021-06-28 to 2021-08-17 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_S112L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_D63G (99%),Spike_L452R (99%),NSP14_A100S (98%),Spike_D950N (98%),Spike_G142D (91%) |
| **608560** | 569/818 | USA (86%), Mexico (10%), Sweden (1%), Israel (1%), Japan (1%) | North America (96%) | 2021-04-15 to 2021-08-20 | **Active** | B.1.617.2 (99%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP6_T181I (100%),Spike_R158del (100%),Spike_F157del (100%),NSP6_V149A (100%),Spike_E156G (100%),Spike_A222V (100%),NSP4_A446V (100%),NSP3_P822L (99%),Spike_V1264L (99%),Spike_D950N (98%),NS6_K48N (97%),Spike_G142D (86%) |
| **608974** | 69/98 | USA (99%), Canada (1%) | North America (100%) | 2021-06-28 to 2021-08-19 | **Active** | B.1.617.2 (85%) | GK (98%) | Delta | Spike_N185D (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_M234I (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_D410G (100%),Spike_T19R (99%),NSP6_T77A (99%),Spike_D950N (98%),NSP6_C197F (97%),Spike_G142D (86%) |
| **609232** | 23/28 | United Kingdom (93%), USA (7%) | Europe (93%) | 2021-06-10 to 2021-08-17 | **Active** | B.1.617.2 (86%) | GK (96%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (93%),NSP3_A1711V (86%) |
| **609320** | 106/287 | United Kingdom (100%) | Europe (100%) | 2021-06-29 to 2021-08-23 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_K97N (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%) |
| **609360** | 167/388 | United Kingdom (100%), USA (0%) | Europe (100%) | 2021-05-23 to 2021-08-23 | **Active** | AY.5 (85%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1469S (100%),NSP12_T870I (100%),NSP2_V126F (100%) |
| **609482** | 59/167 | United Kingdom (82%), USA (11%), Germany (5%), Japan (1%), Indonesia (1%) | Europe (87%) | 2021-05-01 to 2021-08-21 | **Active** | AY.6 (86%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (94%),NSP2_D511N (86%),NSP13_E365D (86%),NSP3_N737D (83%) |
| **609592** | 59/112 | United Kingdom (81%), Portugal (12%), USA (3%), France (1%), Norway (1%) | Europe (97%) | 2021-04-28 to 2021-08-21 | **Active** | B.1.617.2 (65%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS3_A110S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),Spike_D950N (99%),Spike_T95I (97%),Spike_G142D (88%),NSP3_A1711V (72%) |
| **609660** | 66/178 | United Kingdom (100%) | Europe (100%) | 2021-06-29 to 2021-08-22 | **Active** | B.1.617.2 (60%) | GK (99%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),NSP4_A457S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP13_P77L (99%),N_G215C (97%) |
| **609889** | 62/164 | United Kingdom (100%) | Europe (100%) | 2021-05-30 to 2021-08-23 | **Active** | AY.4 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_A1078S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_C197F (94%) |
| **610043** | 44/93 | United Kingdom (100%) | Europe (100%) | 2021-06-22 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_S83L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **610290** | 34/56 | USA (98%), Denmark (2%) | North America (98%) | 2021-06-29 to 2021-08-18 | **Active** | B.1.617.2 (96%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_A879S (100%),NSP12_P323L (100%),NSP16_D269E (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **610328** | 54/91 | USA (99%), Germany (1%) | North America (99%) | 2021-06-21 to 2021-08-17 | **Active** | B.1.617.2 (85%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NS6_E46V (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),NSP12_L131S (91%),NSP2_T531A (91%),Spike_G142D (80%) |
| **610403** | 50/202 | Denmark (82%), France (5%), Netherlands (3%), Germany (3%), Norway (2%) | Europe (100%) | 2021-06-28 to 2021-08-17 | **Active** | B.1.617.2 (92%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),NSP4_A380V (99%),Spike_D950N (99%),Spike_G142D (92%) |
| **610617** | 75/248 | Netherlands (77%), Germany (6%), Curacao (3%), Switzerland (3%), USA (2%) | Europe (93%) | 2021-06-22 to 2021-08-18 | **Active** | B.1.617.2 (90%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP3_S454R (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_V485I (100%),N_D377Y (100%),M_I82T (100%),Spike_T719I (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_G142D (91%) |
| **610619** | 2699/5213 | United Kingdom (41%), Netherlands (21%), Denmark (8%), Sweden (4%), Germany (4%) | Europe (92%) | 2021-03-16 to 2021-08-24 | **Active** | B.1.617.2 (85%) | GK (99%) | Delta | N_R203M (100%),M_I82T (100%),Spike_D614G (100%),NS3_S26L (100%),NS7a_T120I (100%),N_D377Y (100%),NSP12_P323L (100%),NS7a_V82A (100%),Spike_P681R (100%),N_D63G (100%),Spike_T19R (100%),NSP13_P77L (100%),Spike_T478K (100%),Spike_L452R (100%),NSP12_G671S (100%),Spike_F157del (99%),Spike_R158del (99%),Spike_E156G (99%),Spike_D950N (99%),NSP3_P822L (98%),NSP4_A446V (98%),NSP6_T181I (98%),Spike_A222V (98%),NSP6_V149A (98%),Spike_G142D (91%) |
| **611238** | 25/60 | France (20%), Netherlands (13%), Turkey (12%), Sweden (8%), Slovenia (8%) | Europe (98%) | 2021-06-17 to 2021-08-27 | **Active** | B.1.617.2 (65%) | GK (98%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (98%),Spike_D950N (98%),N_Q9L (98%),Spike_T478K (98%),Spike_G142D (67%) |
| **611360** | 135/163 | Mexico (56%), USA (36%), United Kingdom (3%), Finland (1%), France (1%) | North America (93%) | 2021-04-21 to 2021-08-17 | **Active** | B.1.617.2 (91%) | GK (98%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS6_K48N (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_D63G (98%),Spike_N1074S (98%),Spike_V1264L (96%),Spike_D950N (94%) |
| **611413** | 95/171 | USA (99%), Portugal (1%), Ecuador (1%) | North America (99%) | 2021-06-06 to 2021-08-17 | **Active** | AY.4 (96%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (99%),NS7a_T120I (99%),Spike_D950N (96%),NSP3_D410G (94%),NSP6_C197F (91%),NSP2_E587K (80%) |
| **611678** | 175/306 | Spain (50%), Denmark (13%), Italy (5%), France (5%), Netherlands (4%) | Europe (94%) | 2021-05-23 to 2021-08-24 | **Active** | B.1.617.2 (96%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),Spike_T95I (100%),NSP2_V485I (100%),NS7b_T40I (100%),NSP3_P1469S (100%),Spike_D950N (99%),Spike_G142D (96%) |
| **611738** | 32/121 | United Kingdom (98%), Sweden (2%) | Europe (100%) | 2021-06-16 to 2021-08-20 | **Active** | AY.4 (83%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP3_A1711V (98%),NSP4_V233I (83%),NSP2_P568L (83%) |
| **611839** | 118/296 | United Kingdom (100%) | Europe (100%) | 2021-06-26 to 2021-08-23 | **Active** | B.1.617.2 (94%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **611924** | 36/83 | United Kingdom (100%) | Europe (100%) | 2021-06-30 to 2021-08-21 | **Active** | AY.4 (100%) | GK (99%) | Delta | Spike_R21I (100%),N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **611934** | 28/82 | United Kingdom (99%), Sweden (1%) | Europe (100%) | 2021-06-29 to 2021-08-22 | **Active** | AY.4 (100%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),NSP2_A302V (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP13_P504L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_V36F (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **611935** | 226/386 | United Kingdom (99%), Italy (1%), Sweden (0%) | Europe (100%) | 2021-06-07 to 2021-08-22 | **Active** | AY.4 (71%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),Spike_V70F (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP14_D324N (99%) |
| **611936** | 28/46 | United Kingdom (98%), Netherlands (2%) | Europe (100%) | 2021-06-30 to 2021-08-18 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_A623S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **612087** | 62/89 | United Kingdom (91%), Denmark (3%), France (2%), Belgium (1%), USA (1%) | Europe (98%) | 2021-05-30 to 2021-08-23 | **Active** | B.1.617.2 (88%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_G142D (99%),NSP3_A1711V (91%) |
| **612342** | 28/101 | United Kingdom (99%), Switzerland (1%) | Europe (100%) | 2021-06-09 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),Spike_D950N (99%) |
| **612400** | 60/112 | United Kingdom (98%), Germany (2%) | Europe (100%) | 2021-05-29 to 2021-08-22 | **Active** | AY.4 (93%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP12_M463I (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP14_A394V (99%),NS7a_S83L (95%) |
| **612509** | 42/52 | Spain (27%), France (21%), Germany (13%), Switzerland (12%), United Kingdom (12%) | Europe (98%) | 2021-06-10 to 2021-08-18 | **Active** | AY.12 (77%) | GK (100%) | Delta | NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NS3_P240L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_G215C (98%),Spike_T95I (98%),N_R203M (98%),NS7a_V82A (98%),N_D377Y (98%),Spike_D950N (98%),NSP4_V167L (98%),NSP14_A394V (96%),N_D63G (92%),NSP2_V485I (75%) |
| **612657** | 296/576 | Netherlands (74%), France (7%), Germany (5%), Denmark (4%), USA (2%) | Europe (94%) | 2021-06-30 to 2021-08-23 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),NSP7_L56F (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_G215C (100%),Spike_P681R (100%),N_S327L (100%),NSP4_T492I (100%),NS7b_T40I (100%),Spike_L452R (100%),N_W330L (100%),NSP3_T936N (99%),NS3_V88L (99%),Spike_D950N (99%),Spike_G142D (96%) |
| **612660** | 400/1006 | Netherlands (78%), Denmark (14%), Germany (3%), Aruba (3%), United Kingdom (1%) | Europe (96%) | 2021-06-14 to 2021-08-21 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS7a_F46L (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T19R (100%),NSP6_T77A (100%),Spike_T95I (99%),Spike_D950N (99%),Spike_G142D (97%) |
| **612671** | 133/243 | USA (91%), Italy (8%), Mexico (0%), Germany (0%), United Kingdom (0%) | North America (91%) | 2021-06-30 to 2021-08-19 | **Active** | AY.4 (98%) | GK (100%) | Delta | N_G215C (100%),NS8_L60F (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NSP12_F192V (100%),N_D377Y (100%),NSP13_H164Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),NS7a_V82A (100%),NS7a_T120I (100%),NSP3_A1736V (100%),NSP3_H1307Y (100%),N_R203M (99%) |
| **612956** | 116/190 | USA (98%), Denmark (2%) | North America (98%) | 2021-06-30 to 2021-08-21 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),Spike_D950N (98%),Spike_G142D (92%) |
| **612963** | 71/113 | France (57%), Switzerland (16%), Germany (8%), United Kingdom (5%), Belgium (4%) | Europe (99%) | 2021-06-30 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_L41F (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_A65S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (96%),Spike_P251L (96%),NSP3_E906D (96%),NSP1_E87D (96%) |
| **613192** | 29/35 | USA (86%), Denmark (11%), Switzerland (3%) | North America (86%) | 2021-06-30 to 2021-08-23 | **Active** | B.1.617.2 (63%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS8_W45C (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (97%),NSP12_F192V (86%),NSP3_A1736V (83%),Spike_G142D (80%) |
| **613794** | 119/246 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-07 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_P207S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1469S (100%),NSP12_P323L (100%),Spike_T95I (99%),Spike_G142D (99%),NS7a_T120I (98%) |
| **613825** | 35/59 | United Kingdom (98%), Denmark (2%) | Europe (100%) | 2021-07-01 to 2021-08-21 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NS3_G100C (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP4_V293I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **613847** | 61/132 | United Kingdom (99%), Netherlands (1%) | Europe (100%) | 2021-05-30 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_V1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **613858** | 38/122 | Denmark (98%), Sweden (1%), Germany (1%) | Europe (100%) | 2021-07-01 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),NS3_T176I (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%) |
| **613859** | 240/396 | France (41%), Italy (18%), Denmark (8%), Germany (6%), United Kingdom (5%) | Europe (95%) | 2021-04-29 to 2021-08-22 | **Active** | B.1.617.2 (72%) | GK (97%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_R118G (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),Spike_P681R (100%),NSP14_A394V (100%),NSP6_T77A (99%),NSP2_A318V (96%),NSP2_K81N (96%),Spike_D950N (96%),Spike_G142D (72%) |
| **614016** | 42/69 | United Kingdom (100%) | Europe (100%) | 2021-07-01 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_S1230F (96%) |
| **614145** | 96/176 | United Kingdom (99%), Germany (1%) | Europe (100%) | 2021-06-25 to 2021-08-22 | **Active** | AY.4 (100%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP14_A85S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_G142D (99%),NS7a_T120I (99%) |
| **614150** | 52/111 | United Kingdom (75%), Germany (16%), USA (5%), Denmark (3%), Norway (1%) | Europe (95%) | 2021-07-01 to 2021-08-22 | **Active** | AY.6 (87%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),Spike_G142D (97%),NSP3_N737D (95%),NSP2_D511N (95%),NSP13_E365D (95%) |
| **614214** | 42/82 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-06-04 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_F1397L (99%),Spike_G142D (98%),NSP3_Y718C (82%) |
| **614254** | 30/53 | Denmark (51%), Italy (30%), United Kingdom (9%), Sweden (9%) | Europe (100%) | 2021-01-08 to 2021-08-26 | **Active** | AY.4 (85%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (98%),Spike_G142D (98%),NSP2_S36F (91%) |
| **614338** | 62/80 | Italy (38%), Spain (21%), Germany (20%), Switzerland (12%), France (5%) | Europe (99%) | 2021-05-13 to 2021-08-26 | **Active** | AY.12 (70%) | GK (96%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP14_I298V (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_S1252F (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NSP13_H164Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP13_P77L (100%),NSP4_A446V (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_A222V (99%),NSP3_P822L (99%),Spike_E156G (98%),Spike_T478K (98%),Spike_D950N (89%) |
| **614380** | 23/66 | United Kingdom (100%) | Europe (100%) | 2021-07-01 to 2021-08-22 | **Active** | AY.4 (100%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **614578** | 61/107 | United Kingdom (89%), USA (7%), Indonesia (2%), Israel (1%), Japan (1%) | Europe (89%) | 2021-04-06 to 2021-08-22 | **Active** | B.1.617.2 (78%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS8_V62L (99%),Spike_T19R (99%),NSP3_A1711V (96%),Spike_T95I (96%),Spike_G142D (94%) |
| **614838** | 126/244 | USA (70%), Aruba (24%), Sint Maarten (3%), Guatemala (1%), Bonaire (1%) | North America (74%) | 2021-07-01 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_C197F (100%),NS7b_T40I (100%),NSP13_P77L (100%),NS3_L101I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_D410G (100%),Spike_D950N (100%),Spike_G142D (95%) |
| **614976** | 56/101 | Netherlands (32%), Sweden (24%), USA (13%), France (9%), Denmark (8%) | Europe (87%) | 2021-06-30 to 2021-08-18 | **Active** | B.1.617.2 (85%) | GK (97%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_E1207D (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_G142D (89%),NSP16_V288F (87%),NSP3_A1711V (87%),NSP2_A159V (87%) |
| **615161** | 117/196 | Netherlands (57%), Germany (16%), Belgium (10%), Denmark (9%), Italy (3%) | Europe (98%) | 2021-07-01 to 2021-08-20 | **Active** | B.1.617.2 (74%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),NS7a_V82A (100%),NSP14_T16I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP12_R197Q (100%),NSP13_P77L (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_P681R (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_A222V (99%),Spike_E156G (99%),NSP4_A446V (99%),NSP3_P822L (99%),Spike_D950N (98%),Spike_G142D (74%) |
| **615323** | 43/68 | USA (96%), Italy (4%) | North America (96%) | 2021-06-25 to 2021-08-19 | **Active** | AY.3 (99%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_D210A (100%),NSP6_I162V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_V70I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (99%),Spike_D950N (97%) |
| **615868** | 34/73 | United Kingdom (100%) | Europe (100%) | 2021-07-02 to 2021-08-23 | **Active** | B.1.617.2 (96%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NSP16_S2N (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T678I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **615887** | 214/415 | United Kingdom (99%), Japan (0%), Portugal (0%) | Europe (100%) | 2021-06-19 to 2021-08-22 | **Active** | AY.4 (93%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS3_S26L (100%),Spike_G142D (99%),NSP15_T114A (93%) |
| **615995** | 25/31 | United Kingdom (97%), USA (3%) | Europe (97%) | 2021-05-21 to 2021-08-21 | **Active** | AY.4 (71%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_T859I (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (94%) |
| **616047** | 68/106 | Italy (96%), Switzerland (3%), Japan (1%) | Europe (99%) | 2021-07-02 to 2021-08-19 | **Active** | AY.12 (68%) | GK (97%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_L139F (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),NSP6_L37F (95%) |
| **616754** | 59/238 | Netherlands (57%), Sint Maarten (17%), United Kingdom (5%), Germany (5%), Belgium (4%) | Europe (81%) | 2021-06-29 to 2021-08-21 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_D950N (99%),NSP14_T16I (96%),NSP12_R197Q (96%),Spike_G142D (92%) |
| **616885** | 71/135 | USA (99%), Aruba (1%) | North America (99%) | 2021-06-23 to 2021-08-20 | **Active** | B.1.617.2 (98%) | GK (99%) | Delta | Spike_V1264L (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_N1074S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),N_T362I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP3_P822L (99%),NS6_K48N (99%),Spike_G142D (82%) |
| **617262** | 68/109 | France (56%), Belgium (21%), Netherlands (5%), Switzerland (4%), United Kingdom (4%) | Europe (96%) | 2021-05-11 to 2021-08-19 | **Active** | AY.12 (58%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS3_Q116H (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T19R (99%),Spike_D950N (99%),NSP3_P340L (93%),NSP15_L162F (70%),NSP12_A656V (70%) |
| **617422** | 29/153 | United Kingdom (97%), France (1%), Spain (1%), Portugal (1%), South Korea (1%) | Europe (99%) | 2021-07-03 to 2021-08-23 | **Active** | AY.5 (97%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (97%) |
| **617474** | 86/198 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-06-22 to 2021-08-23 | **Active** | B.1.617.2 (88%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),N_D402Y (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP12_T710I (88%),NSP6_A2V (88%) |
| **617541** | 55/129 | United Kingdom (98%), Japan (1%), Denmark (1%), Germany (1%) | Europe (99%) | 2021-05-05 to 2021-08-23 | **Active** | B.1.617.2 (88%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),NSP3_A1711V (98%),Spike_G142D (98%) |
| **617879** | 30/52 | Belgium (69%), Italy (13%), Denmark (10%), Netherlands (2%), Sweden (2%) | Europe (100%) | 2021-07-03 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (96%),Spike_T791I (96%) |
| **617967** | 280/505 | USA (99%), Aruba (0%), Japan (0%), Netherlands (0%), United Kingdom (0%) | North America (99%) | 2021-06-23 to 2021-08-20 | **Active** | B.1.617.2 (99%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NSP13_A296S (100%),N_D377Y (100%),NS8_Y31F (100%),NSP6_T77A (100%),Spike_L452R (100%),Spike_D950N (99%),Spike_G142D (95%) |
| **618077** | 36/65 | Italy (49%), France (37%), Netherlands (3%), Denmark (3%), Switzerland (3%) | Europe (98%) | 2021-07-03 to 2021-08-20 | **Active** | B.1.617.2 (75%) | GK (97%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_V5I (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_V485I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_G142D (75%) |
| **618449** | 30/52 | United Kingdom (96%), Denmark (4%) | Europe (100%) | 2021-06-27 to 2021-08-21 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),N_Q9H (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **618695** | 141/410 | United Kingdom (100%) | Europe (100%) | 2021-07-04 to 2021-08-23 | **Active** | AY.4 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_H613Y (100%),Spike_G142D (100%),NSP13_I79V (65%) |
| **618892** | 41/70 | United Kingdom (100%) | Europe (100%) | 2021-06-24 to 2021-08-18 | **Active** | B.1.617.2 (84%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP2_E197A (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NS8_Q72H (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS6_L40I (99%),Spike_D950N (97%),Spike_G142D (93%) |
| **618944** | 43/86 | United Kingdom (100%) | Europe (100%) | 2021-07-04 to 2021-08-22 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NSP2_S196L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A664V (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP3_S1699P (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP2_S430L (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **619049** | 36/49 | United Kingdom (100%) | Europe (100%) | 2021-07-04 to 2021-08-21 | **Active** | B.1.617.2 (90%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),NSP1_E41K (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS3_Y91H (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_S67F (100%),Spike_T478K (100%),NSP14_A85S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%) |
| **619219** | 117/202 | United Kingdom (67%), Denmark (10%), Belgium (4%), Switzerland (3%), Turkey (3%) | Europe (98%) | 2021-05-17 to 2021-08-23 | **Active** | AY.4 (51%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_L85F (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP3_A488S (100%),Spike_T95I (99%),NSP4_V167L (99%),Spike_G142D (97%),NSP3_A1711V (67%) |
| **619280** | 61/159 | United Kingdom (58%), Denmark (33%), Switzerland (3%), Germany (2%), Sweden (1%) | Europe (100%) | 2021-06-20 to 2021-08-23 | **Active** | AY.4 (96%) | GK (97%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_A446V (100%),NS3_S26L (100%),NSP13_V348L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP3_S1285F (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (99%),NS3_T269S (99%),Spike_A1078S (94%),Spike_G142D (75%) |
| **619326** | 64/117 | USA (99%), United Kingdom (1%) | North America (99%) | 2021-07-02 to 2021-08-22 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NS3_L108F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP2_E587K (100%),NSP6_C197F (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_D410G (100%),Spike_G142D (93%) |
| **620014** | 125/202 | United Kingdom (69%), USA (14%), Italy (9%), Brazil (2%), France (2%) | Europe (82%) | 2021-04-19 to 2021-08-22 | **Active** | AY.6 (78%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_P681R (100%),NS7a_T120I (98%),Spike_D950N (97%),Spike_G142D (84%),NSP2_D511N (82%),NSP13_E365D (82%) |
| **620147** | 25/46 | United Kingdom (100%) | Europe (100%) | 2021-06-23 to 2021-08-21 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),NSP5_R298G (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (98%) |
| **620187** | 152/413 | United Kingdom (100%) | Europe (100%) | 2021-05-17 to 2021-08-23 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (100%) |
| **620249** | 82/137 | United Kingdom (82%), France (4%), USA (3%), Netherlands (2%), Spain (2%) | Europe (96%) | 2021-05-31 to 2021-08-22 | **Active** | B.1.617.2 (81%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),NS3_G100C (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_R158del (99%),NS7a_V82A (99%),Spike_F157del (99%),Spike_E156G (99%),NSP6_T77A (99%),NSP4_V167L (99%),NSP3_A1711V (96%),Spike_T95I (95%),Spike_G142D (94%),NSP2_T223I (82%) |
| **620362** | 32/64 | United Kingdom (98%), Slovakia (2%) | Europe (100%) | 2021-06-23 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS7a_L5I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_Q493E (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_A623V (81%) |
| **620461** | 29/65 | United Kingdom (100%) | Europe (100%) | 2021-07-05 to 2021-08-22 | **Active** | AY.4 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),N_A12V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **620509** | 30/57 | United Kingdom (98%), Denmark (2%) | Europe (100%) | 2021-07-05 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP4_I27L (63%) |
| **620983** | 300/515 | United Kingdom (97%), USA (2%), Japan (0%), Poland (0%), Mexico (0%) | Europe (98%) | 2021-05-24 to 2021-08-23 | **Active** | AY.4 (83%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NS7a_H73Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),Spike_T19R (100%),NSP3_A1711V (99%),NSP12_D879N (97%) |
| **621011** | 36/66 | United Kingdom (95%), USA (5%) | Europe (95%) | 2021-07-05 to 2021-08-22 | **Active** | B.1.617.2 (94%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_K67N (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_T95I (97%),Spike_G142D (97%),NSP3_A1711V (95%),NSP3_S1230F (92%) |
| **621412** | 334/585 | USA (99%), Sint Maarten (0%), United Kingdom (0%) | North America (100%) | 2021-06-10 to 2021-08-20 | **Active** | B.1.617.2 (93%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NS8_P36S (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS7a_V82A (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),NSP3_P1469S (100%),NSP4_V167L (100%),Spike_D950N (100%),N_G215C (99%),Spike_G142D (92%) |
| **622318** | 39/69 | Italy (25%), France (23%), Denmark (19%), Switzerland (17%), Germany (6%) | Europe (97%) | 2021-07-05 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_A105S (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP3_E906D (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (100%),N_D63G (99%),Spike_P251L (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),NS8_A65S (94%) |
| **622439** | 24/39 | Germany (56%), Belgium (18%), Spain (13%), USA (5%), Bosnia and Herzegovina (3%) | Europe (95%) | 2021-06-28 to 2021-08-23 | **Active** | B.1.617.2 (100%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1469S (97%),N_R203M (97%),NSP12_P323L (95%),N_Q9L (92%),NSP12_L838I (92%) |
| **622446** | 82/106 | France (23%), United Kingdom (20%), USA (14%), Belgium (8%), Switzerland (7%) | Europe (74%) | 2021-06-23 to 2021-08-27 | **Active** | B.1.617.2 (69%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP4_C296F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q677H (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_M72I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),N_L230F (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%),Spike_L1265F (98%),Spike_D950N (94%),NS3_ins104P (84%),Spike_G142D (66%) |
| **622724** | 39/63 | United Kingdom (100%) | Europe (100%) | 2021-07-02 to 2021-08-19 | **Active** | AY.4 (100%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_D1318N (100%),NS7a_V82A (100%),NSP3_S1424F (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),N_S193R (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),M_A63V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **622741** | 25/57 | United Kingdom (100%) | Europe (100%) | 2021-07-06 to 2021-08-23 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),N_G18V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_K235T (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **622841** | 47/65 | Portugal (92%), France (5%), United Kingdom (2%), Switzerland (2%) | Europe (100%) | 2021-06-29 to 2021-08-17 | **Active** | AY.12 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),N_A119S (100%),NS3_S26L (100%),Spike_P681R (100%),NS3_A110S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP2_S263F (100%),NS3_G18V (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),NSP5_K90R (100%),NSP16_P215T (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **623309** | 45/136 | France (65%), Netherlands (8%), Switzerland (6%), Italy (5%), Belgium (4%) | Europe (97%) | 2021-06-29 to 2021-08-21 | **Active** | B.1.617.2 (77%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),M_I82T (99%),Spike_D950N (96%),Spike_G142D (77%) |
| **623528** | 83/118 | USA (95%), Germany (3%), United Kingdom (1%), Romania (1%), Switzerland (1%) | North America (95%) | 2021-05-12 to 2021-08-18 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),N_A152S (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_P1228L (99%),NSP3_P1469S (99%),NS7a_V82A (98%),NS7a_T120I (98%),Spike_D950N (98%),NSP6_T77A (97%),Spike_G142D (94%),NSP12_F192V (94%),NSP3_A1736V (94%),NSP2_K81R (77%) |
| **623581** | 146/264 | USA (99%), Denmark (0%), Japan (0%) | North America (99%) | 2021-07-06 to 2021-08-21 | **Active** | B.1.617.2 (93%) | GK (100%) | Delta | N_G215C (100%),NS8_L60F (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP12_F192V (100%),N_D377Y (100%),NSP13_H164Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (100%),NSP3_A1736V (100%),NSP3_H1307Y (100%),N_D63S (98%),Spike_R158del (97%),Spike_F157del (97%),Spike_E156G (97%),Spike_G142D (94%) |
| **624142** | 30/48 | USA (100%) | North America (100%) | 2021-07-06 to 2021-08-18 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NSP13_A296S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS8_Y31F (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **624768** | 27/60 | United Kingdom (97%), Denmark (3%) | Europe (100%) | 2021-07-07 to 2021-08-22 | **Active** | B.1.617.2 (73%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP14_H26Y (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_S92L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_A845S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP9_T24I (60%) |
| **624960** | 130/412 | Denmark (63%), Sweden (15%), United Kingdom (13%), Norway (2%), Germany (1%) | Europe (100%) | 2021-07-07 to 2021-08-24 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (100%),NSP2_A159V (100%),NSP3_A1711V (99%),N_D63G (99%),Spike_G142D (94%) |
| **625034** | 72/160 | United Kingdom (100%) | Europe (100%) | 2021-07-07 to 2021-08-23 | **Active** | AY.4 (60%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP3_P1044L (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP3_E906D (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D216E (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **625055** | 70/153 | United Kingdom (93%), USA (5%), Poland (1%), Denmark (1%), Sweden (1%) | Europe (95%) | 2021-06-29 to 2021-08-22 | **Active** | B.1.617.2 (81%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),Spike_M1237I (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_T120I (98%),NSP3_A1711V (93%) |
| **625590** | 91/138 | Italy (71%), Slovenia (8%), Germany (8%), Switzerland (7%), France (2%) | Europe (100%) | 2021-07-04 to 2021-08-19 | **Active** | AY.7 (79%) | GK (95%) | Delta | NS7a_A79D (100%),N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP9_Q11H (100%),NS3_G18V (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_P681R (99%),NSP12_P323L (99%),M_I82T (99%),NS7a_C58F (99%),N_D63G (97%),Spike_D950N (96%),NSP4_V167L (96%) |
| **625990** | 716/1081 | USA (89%), Australia (6%), New Zealand (5%), United Kingdom (0%), Aruba (0%) | North America (89%) | 2021-05-13 to 2021-08-25 | **Active** | B.1.617.2 (93%) | GK (100%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_V71I (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),N_D377Y (100%),Spike_E156G (100%),NSP6_T77A (100%),NSP12_P323L (100%),NS7a_V82A (100%),Spike_T95I (100%),NSP16_Q238H (99%),Spike_D950N (99%),Spike_G142D (93%) |
| **626214** | 43/87 | France (39%), Germany (32%), Switzerland (15%), Belgium (7%), Italy (2%) | Europe (99%) | 2021-07-07 to 2021-08-19 | **Active** | AY.5 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),N_W330L (100%),NSP12_G671S (100%),NSP3_T936N (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP7_L56F (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP4_T492I (99%),Spike_D950N (99%),NS3_V88L (99%) |
| **626562** | 53/68 | USA (99%), Germany (1%) | North America (99%) | 2021-06-26 to 2021-08-18 | **Active** | AY.4 (100%) | GK (96%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP9_R39K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),M_I82T (99%),Spike_D950N (97%),NSP3_P1469S (97%) |
| **626706** | 73/124 | USA (99%), Switzerland (1%) | North America (99%) | 2021-07-05 to 2021-08-18 | **Active** | B.1.617.2 (90%) | GK (98%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),N_P122L (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),M_I82T (100%),NSP6_V149A (100%),NSP2_E460D (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_V367L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP14_P43L (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_D377Y (99%),Spike_D950N (99%),Spike_G142D (90%) |
| **626857** | 78/142 | Switzerland (82%), Germany (9%), Denmark (4%), United Kingdom (3%), Italy (1%) | Europe (100%) | 2021-07-07 to 2021-08-23 | **Active** | AY.9 (82%) | GK (100%) | Delta | N_R203M (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_T19R (100%),Spike_G1124V (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS7a_V82A (99%),NSP14_T16I (99%),NSP12_R197Q (99%),NSP4_A446V (99%),NSP15_S161I (99%),N_D63G (86%),Spike_D950N (84%) |
| **626908** | 179/300 | Germany (31%), Czech Republic (22%), Netherlands (11%), Italy (10%), Denmark (5%) | Europe (100%) | 2021-06-17 to 2021-08-26 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P192L (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),Spike_D253A (100%),M_I82T (100%),NSP13_P77L (100%),NSP4_A446V (100%),Spike_T478K (100%),Spike_D979E (100%),NSP3_T1830I (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_R203M (100%),Spike_F157del (100%),Spike_A222V (100%),NSP6_V149A (100%),NSP3_P822L (100%),Spike_R158del (99%),Spike_E156G (99%),NS7a_V82A (99%),N_D63G (97%),Spike_D950N (97%) |
| **627856** | 28/58 | Denmark (33%), Netherlands (19%), France (12%), Italy (10%), Switzerland (9%) | Europe (98%) | 2021-07-08 to 2021-08-24 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P251L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_A65S (100%),NS7a_A105S (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP3_E906D (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP1_E87D (100%),N_R203M (98%) |
| **628135** | 58/85 | USA (100%) | North America (100%) | 2021-06-30 to 2021-08-20 | **Active** | B.1.617.2 (86%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_W131C (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NS8_P36S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP13_I334V (99%),NS7a_T120I (99%),Spike_D950N (99%),NSP13_P300L (99%),NSP6_T77A (96%),Spike_G142D (87%) |
| **629066** | 80/165 | Netherlands (24%), Turkey (24%), Denmark (21%), Germany (14%), USA (5%) | Europe (93%) | 2021-06-11 to 2021-08-21 | **Active** | AY.4 (59%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),Spike_T250I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T29A (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (99%),Spike_G142D (99%),Spike_D950N (99%),NS7a_T120I (93%) |
| **629902** | 64/70 | Cambodia (81%), Belgium (7%), South Korea (3%), India (3%), USA (3%) | Asia (87%) | 2021-02-19 to 2021-08-21 | **Active** | B.1.1.7 (100%) | GRY (57%) | Alpha | NS8_Q27stop (100%),NSP3_T183I (100%),NSP3_A890D (100%),Spike_T716I (100%),NS8_K68stop (100%),N_R203K (100%),Spike_A570D (100%),Spike_D1118H (100%),NS8_Y73C (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),NSP12_P323L (100%),Spike_P681H (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),Spike_H69del (99%),N_G204R (99%),Spike_V70del (99%),Spike_Y144del (99%),NSP6_S106del (96%),NSP6_G107del (96%),NSP6_F108del (96%) |
| **629973** | 41/68 | Germany (75%), Switzerland (4%), Belgium (4%), Slovenia (3%), Denmark (3%) | Europe (99%) | 2021-07-01 to 2021-08-20 | **Active** | AY.4 (60%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NSP2_G339S (100%),NSP12_K91R (100%),NS3_S26L (100%),Spike_P681R (100%),NSP12_P227S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP2_K489N (100%),Spike_A262S (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_A119V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_S477I (100%),Spike_G1167V (100%),NS7a_T120I (99%),Spike_D950N (97%),NSP3_A488S (94%) |
| **630316** | 24/48 | USA (100%) | North America (100%) | 2021-07-09 to 2021-08-17 | **Active** | AY.4 (85%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_D138H (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),Spike_G142D (98%),Spike_D950N (98%),NSP6_T77A (98%) |
| **630604** | 29/62 | USA (100%) | North America (100%) | 2021-07-06 to 2021-08-17 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | NS7a_L116F (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP14_S374F (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP3_T1056I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP6_V149A (98%),Spike_T478K (98%),Spike_L452R (98%),Spike_G142D (94%) |
| **631014** | 156/259 | USA (99%), United Kingdom (1%) | North America (99%) | 2021-06-28 to 2021-08-17 | **Active** | AY.3 (100%) | GK (100%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D63G (100%),NSP6_I162V (100%),Spike_D950N (99%),NSP6_T77A (99%),M_I82T (96%),Spike_G142D (91%),NSP5_I259L (73%) |
| **631144** | 35/56 | USA (88%), Mexico (9%), United Kingdom (2%), Denmark (2%) | North America (96%) | 2021-07-09 to 2021-08-18 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_A419V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_V90F (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_V1104L (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (98%),Spike_D950N (95%),Spike_G142D (91%) |
| **631744** | 23/33 | United Kingdom (82%), Denmark (9%), Spain (3%), Singapore (3%), USA (3%) | Europe (94%) | 2021-04-22 to 2021-08-22 | **Active** | B.1.617.2 (85%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (97%),Spike_T95I (94%),NSP3_A1711V (79%) |
| **631943** | 67/101 | France (36%), Netherlands (17%), United Kingdom (12%), Germany (8%), Switzerland (6%) | Europe (93%) | 2021-04-18 to 2021-08-19 | **Active** | AY.4 (59%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_T19R (98%),NSP3_A1711V (65%),Spike_G142D (64%) |
| **632351** | 54/69 | France (20%), Denmark (17%), United Kingdom (10%), Turkey (10%), Germany (10%) | Europe (99%) | 2021-07-08 to 2021-08-23 | **Active** | B.1.617.2 (62%) | GK (100%) | Delta | N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_P207L (99%),Spike_A222V (99%),NSP6_V149A (99%),NSP3_P822L (99%),N_P46S (99%),NSP4_A446V (99%),NSP3_S1285F (99%),Spike_D950N (97%),N_D63G (96%),NSP6_T181I (88%),Spike_G142D (62%) |
| **632558** | 90/135 | USA (96%), Germany (4%), Singapore (1%) | North America (96%) | 2021-06-08 to 2021-08-20 | **Active** | B.1.617.2 (96%) | GK (97%) | Delta | M_S197N (100%),NSP12_G671S (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),NS3_G224C (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_D63G (99%),N_R203M (99%),Spike_D950N (99%),NSP3_P402H (83%),Spike_G142D (77%) |
| **632773** | 596/1518 | Singapore (73%), Indonesia (19%), Japan (3%), South Korea (1%), Netherlands (1%) | Asia (97%) | 2021-04-06 to 2021-08-25 | **Active** | B.1.617.2 (98%) | GK (99%) | Delta | N_G215C (100%),Spike_V1264L (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),NS7b_T40I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),N_R203M (100%),N_D377Y (100%),NSP4_V167L (100%),NSP2_P200L (100%),NS7a_V82A (100%),NSP13_P77L (100%),NSP3_T678I (100%),Spike_D950N (100%),Spike_G142D (98%) |
| **632893** | 24/49 | United Kingdom (100%) | Europe (100%) | 2021-06-21 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NS8_C102F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **633236** | 73/96 | Cambodia (96%), Thailand (2%), South Korea (1%), Japan (1%) | Asia (100%) | 2021-05-18 to 2021-08-22 | **Active** | AY.11 (75%) | GK (99%) | Delta | NS7a_L116F (100%),NSP12_G671S (100%),NSP2_P129L (100%),Spike_P681R (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP4_A446V (100%),NSP15_H234Y (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS3_S26L (99%),NSP13_P77L (99%),N_R203M (98%),N_L139F (98%),NSP6_V149A (98%),N_R385K (92%),NSP3_H1274Y (79%),N_D63G (76%) |
| **633362** | 49/69 | Portugal (81%), United Kingdom (7%), Netherlands (7%), Romania (3%), Curacao (1%) | Europe (99%) | 2021-05-03 to 2021-08-17 | **Active** | AY.12 (81%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q14H (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P323L (99%) |
| **633392** | 70/149 | Turkey (42%), United Kingdom (26%), USA (24%), Denmark (5%), Germany (2%) | Europe (74%) | 2021-05-26 to 2021-08-21 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NS3_W69C (99%),Spike_T19R (99%),Spike_T95I (98%),Spike_G142D (98%),Spike_D950N (98%),NSP6_T77A (97%),NSP4_V167L (80%) |
| **633829** | 116/190 | France (32%), United Kingdom (25%), USA (17%), Switzerland (7%), Belgium (4%) | Europe (78%) | 2021-06-28 to 2021-08-21 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP4_C296F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q677H (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_M72I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP6_T77A (99%),NSP12_P323L (99%),N_R203M (99%),Spike_D950N (97%),NS3_ins104P (95%),Spike_G142D (91%),NSP16_T151I (64%) |
| **634249** | 45/141 | Denmark (94%), Germany (2%), Czech Republic (2%), Japan (1%), Switzerland (1%) | Europe (99%) | 2021-07-12 to 2021-08-17 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP3_E387D (100%),NSP6_T181I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),NSP3_P192L (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),Spike_D253A (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),M_T30I (100%),NSP4_A446V (100%),Spike_T478K (100%),Spike_D979E (100%),NSP3_T1830I (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **634655** | 24/67 | United Kingdom (100%) | Europe (100%) | 2021-07-10 to 2021-08-23 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_P1162S (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **635021** | 691/905 | USA (34%), Switzerland (9%), Turkey (8%), Brazil (7%), France (7%) | Europe (52%) | 2021-04-04 to 2021-08-24 | **Active** | AY.4 (75%) | GK (97%) | Delta | Spike_P681R (100%),Spike_T478K (100%),NS3_S26L (100%),Spike_T19R (100%),N_D377Y (100%),NSP13_P77L (100%),Spike_D614G (100%),N_R203M (100%),Spike_R158del (100%),Spike_F157del (100%),NSP12_G671S (100%),NS7a_T120I (100%),Spike_E156G (100%),Spike_L452R (100%),NSP12_P323L (99%),NSP6_T77A (99%),M_I82T (99%),NSP14_A394V (99%),Spike_D950N (99%),N_G215C (99%),NS7a_V82A (99%),NSP4_T492I (99%),NSP3_P1228L (98%),NSP3_P1469S (98%),NS7b_T40I (98%),NSP3_A488S (98%),N_D63G (97%),NSP4_V167L (93%) |
| **635109** | 73/164 | France (77%), Belgium (13%), Denmark (4%), Italy (2%), Germany (2%) | Europe (100%) | 2021-07-04 to 2021-08-19 | **Active** | B.1.617.2 (96%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_V77F (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP14_R289H (100%),NS3_D27Y (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_E156G (99%),NSP12_Q822H (99%),NSP14_A394V (99%),Spike_D950N (99%),Spike_G142D (96%),NS3_A31T (82%) |
| **636852** | 24/46 | United Kingdom (100%) | Europe (100%) | 2021-07-05 to 2021-08-21 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP14_I42T (83%),NSP13_Y324C (80%) |
| **636866** | 54/113 | United Kingdom (99%), Netherlands (1%) | Europe (100%) | 2021-07-12 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (96%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **636994** | 92/150 | United Kingdom (91%), Belgium (3%), France (1%), Netherlands (1%), USA (1%) | Europe (98%) | 2021-05-24 to 2021-08-22 | **Active** | AY.4 (72%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_G238C (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (99%),Spike_G142D (97%),NSP3_A1711V (89%),Spike_T95I (89%) |
| **638058** | 48/76 | France (49%), Switzerland (26%), Portugal (12%), Germany (7%), Belgium (4%) | Europe (99%) | 2021-07-06 to 2021-08-17 | **Active** | AY.12 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NSP4_C296F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_Q677H (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP14_M72I (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),NS3_ins104P (97%) |
| **638504** | 23/52 | USA (100%) | North America (100%) | 2021-07-05 to 2021-08-18 | **Active** | B.1.617.2 (94%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_A208S (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),NSP2_G339S (100%),NS3_P104S (100%),NS3_S26L (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_P681R (98%),Spike_D950N (98%),Spike_G142D (94%) |
| **639587** | 98/231 | United Kingdom (100%) | Europe (100%) | 2021-07-14 to 2021-08-23 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS3_E266K (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%),NSP3_V393I (99%) |
| **639993** | 35/54 | United Kingdom (100%) | Europe (100%) | 2021-06-28 to 2021-08-22 | **Active** | B.1.617.2 (87%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),E_L73F (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (98%),NS7a_T120I (96%) |
| **640004** | 107/270 | United Kingdom (100%), Switzerland (0%) | Europe (100%) | 2021-07-14 to 2021-08-23 | **Active** | B.1.617.2 (93%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP12_L270F (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NS3_W128L (100%),NSP3_P1469S (100%),Spike_T478K (100%),N_A12S (100%),NSP6_T77A (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),NSP12_P323L (100%),NSP3_E177K (78%) |
| **640044** | 39/75 | United Kingdom (100%) | Europe (100%) | 2021-07-11 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),Spike_T51I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_S1230F (99%) |
| **640170** | 51/87 | Italy (31%), Denmark (22%), Netherlands (16%), Germany (11%), Greece (6%) | Europe (95%) | 2021-07-09 to 2021-08-23 | **Active** | B.1.617.2 (90%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_G18V (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_A79D (99%),NSP9_Q11H (99%),NS7a_V82A (99%),NSP3_P1469S (99%),NS7a_C58F (98%),Spike_G142D (92%) |
| **640388** | 43/60 | USA (67%), United Kingdom (20%), Germany (3%), Brazil (3%), Italy (3%) | North America (67%) | 2021-06-08 to 2021-08-20 | **Active** | AY.4 (50%) | GK (98%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (95%) |
| **641384** | 29/54 | USA (98%), United Kingdom (2%) | North America (98%) | 2021-07-08 to 2021-08-18 | **Active** | B.1.617.2 (89%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_Q940H (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_S112L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NS3_E239Q (100%),NSP5_L220F (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (98%),Spike_G142D (89%) |
| **642142** | 36/81 | United Kingdom (100%) | Europe (100%) | 2021-07-09 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NSP6_K263N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NS7a_A106S (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **642275** | 33/202 | United Kingdom (100%) | Europe (100%) | 2021-07-15 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **642558** | 55/117 | United Kingdom (100%) | Europe (100%) | 2021-05-06 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS8_V33F (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP4_V233I (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NS3_S180F (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP6_L37F (100%),Spike_L452R (100%),NSP4_V167L (99%) |
| **642906** | 40/59 | Russia (22%), France (12%), Turkey (12%), Germany (8%), Poland (7%) | Europe (95%) | 2021-06-05 to 2021-08-21 | **Active** | B.1.617.2 (95%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NSP2_K489N (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_P45L (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R158del (97%),Spike_F157del (97%),Spike_E156G (97%),Spike_G142D (95%) |
| **643372** | 81/131 | USA (100%) | North America (100%) | 2021-07-05 to 2021-08-18 | **Active** | B.1.617.2 (82%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS7a_K2E (100%),Spike_T478K (100%),NS7a_A8T (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A488S (99%),NSP1_F143del (99%),NSP1_K141del (99%),NSP1_S142del (99%),NSP6_T77A (99%),NSP2_S36Y (99%),Spike_D950N (98%),Spike_G142D (83%),NSP14_T372I (71%),NSP10_T101I (70%) |
| **644513** | 70/139 | United Kingdom (99%), Israel (1%), Japan (1%) | Europe (99%) | 2021-06-17 to 2021-08-22 | **Active** | AY.4 (97%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),NSP4_H36Y (100%),Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NS7a_H73Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP12_D879N (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%) |
| **644762** | 47/120 | United Kingdom (100%) | Europe (100%) | 2021-07-13 to 2021-08-23 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_I387V (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),N_V270L (97%) |
| **645102** | 56/85 | United Kingdom (100%) | Europe (100%) | 2021-06-18 to 2021-08-21 | **Active** | AY.4 (100%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),Spike_D253G (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (96%),NSP4_M366I (66%) |
| **648171** | 87/131 | Denmark (42%), Germany (29%), Switzerland (12%), North Macedonia (8%), Sweden (4%) | Europe (100%) | 2021-06-14 to 2021-08-24 | **Active** | B.1.617.2 (76%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP16_D179G (99%),NS8_I88T (98%),Spike_D950N (90%),Spike_G142D (76%) |
| **648298** | 52/78 | USA (23%), Greece (15%), United Kingdom (13%), Germany (13%), Denmark (10%) | Europe (77%) | 2021-06-26 to 2021-08-23 | **Active** | B.1.617.2 (83%) | GK (99%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_A318V (100%),NSP12_E744D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_G181V (100%),NSP6_T77A (100%),NS7a_R118G (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A488S (99%),Spike_D950N (99%),N_G215C (95%),Spike_G142D (83%) |
| **648373** | 39/80 | Denmark (64%), Germany (19%), Sweden (6%), Netherlands (5%), Norway (1%) | Europe (99%) | 2021-07-18 to 2021-08-21 | **Active** | B.1.617.2 (89%) | GK (96%) | Delta | NS7a_E41D (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_P45L (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (94%),Spike_G142D (89%) |
| **648817** | 88/118 | Australia (55%), USA (44%), Portugal (1%) | Oceania (55%) | 2021-05-13 to 2021-08-19 | **Active** | AY.12 (92%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_V71I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),NSP16_Q238H (99%),NSP14_A394V (99%),Spike_D950N (97%) |
| **648852** | 85/152 | United Kingdom (98%), Belgium (1%), Netherlands (1%) | Europe (100%) | 2021-06-22 to 2021-08-23 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP16_L111F (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **650708** | 128/203 | Sweden (32%), Finland (17%), Denmark (16%), Switzerland (10%), Germany (9%) | Europe (98%) | 2021-07-01 to 2021-08-21 | **Active** | B.1.617.2 (64%) | GK (99%) | Delta | N_G215C (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),NSP2_K81N (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP14_A394V (100%),Spike_R158del (99%),Spike_F157del (99%),Spike_E156G (99%),NSP3_N1785S (95%),NSP14_D258A (95%),NS7a_P45L (92%),Spike_D950N (87%),N_D63G (82%),Spike_G142D (64%) |
| **650983** | 37/59 | United Kingdom (100%) | Europe (100%) | 2021-06-28 to 2021-08-22 | **Active** | AY.4 (100%) | GK (98%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_D377Y (98%) |
| **653698** | 41/360 | Denmark (99%), France (1%), United Kingdom (0%) | Europe (100%) | 2021-07-13 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),N_Q9L (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_L838I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (100%),NS7a_T120I (100%),Spike_G142D (99%) |
| **653782** | 21/77 | United Kingdom (99%), France (1%) | Europe (100%) | 2021-07-20 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_E554G (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **655997** | 32/75 | United Kingdom (95%), USA (4%), Denmark (1%) | Europe (96%) | 2021-06-27 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP3_A1711V (96%),Spike_T95I (96%) |
| **657112** | 30/37 | United Kingdom (81%), USA (11%), Guatemala (5%), Turkey (3%) | Europe (84%) | 2021-05-25 to 2021-08-22 | **Active** | B.1.617.2 (81%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (95%),NSP3_A1711V (78%) |
| **657257** | 58/133 | United Kingdom (100%) | Europe (100%) | 2021-07-02 to 2021-08-23 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NS7a_I88T (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (98%),NSP3_G1389S (88%),N_P199S (81%) |
| **657263** | 21/23 | United Kingdom (87%), USA (13%) | Europe (87%) | 2021-05-17 to 2021-08-18 | **Active** | B.1.617.2 (74%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS8_E106stop (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (91%),NSP3_A1711V (87%),Spike_T95I (87%) |
| **659681** | 52/113 | United Kingdom (99%), Japan (1%) | Europe (99%) | 2021-05-05 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_Q185H (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (99%) |
| **660362** | 103/228 | United Kingdom (100%) | Europe (100%) | 2021-07-22 to 2021-08-23 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_A50S (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G34E (99%),NSP2_S36A (99%),NSP3_N208T (98%) |
| **660387** | 34/84 | United Kingdom (100%) | Europe (100%) | 2021-07-01 to 2021-08-22 | **Active** | AY.5 (87%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%) |
| **660395** | 135/219 | USA (86%), United Kingdom (13%), Sint Maarten (1%) | North America (87%) | 2021-06-24 to 2021-08-22 | **Active** | B.1.617.2 (93%) | GK (100%) | Delta | N_G215C (100%),Spike_P809S (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_D950N (100%),NS7a_V82A (99%),Spike_G142D (95%),NSP4_A92V (88%),NSP14_A323S (87%),NSP9_T24I (79%) |
| **661986** | 85/236 | United Kingdom (100%), Switzerland (0%) | Europe (100%) | 2021-07-21 to 2021-08-22 | **Active** | AY.4 (100%) | GK (100%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS3_V48F (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_K854N (100%),Spike_E156G (100%),NS8_A65V (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1769V (100%),Spike_G142D (100%),NSP2_P181L (100%),N_G215C (99%) |
| **662204** | 48/140 | United Kingdom (96%), Denmark (4%) | Europe (100%) | 2021-07-23 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NSP4_A22V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **662451** | 37/48 | United Kingdom (88%), Belgium (4%), Japan (2%), Mexico (2%), USA (2%) | Europe (94%) | 2021-05-07 to 2021-08-19 | **Active** | B.1.617.2 (50%) | GK (100%) | Delta | N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (98%),Spike_D950N (98%),NS7b_T40I (98%),Spike_G142D (96%),Spike_T95I (94%),NSP3_A1711V (81%) |
| **662510** | 26/47 | United Kingdom (100%) | Europe (100%) | 2021-07-23 to 2021-08-23 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_A1087S (100%),NSP14_G481C (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS3_V29del (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),E_P71S (100%),NSP14_L157F (94%) |
| **662536** | 52/103 | United Kingdom (97%), Portugal (2%), Sweden (1%) | Europe (100%) | 2021-06-19 to 2021-08-23 | **Active** | B.1.617.2 (95%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_T51I (99%),NSP3_S1230F (99%),NSP6_T77A (98%),Spike_G142D (95%),NS3_L129F (92%) |
| **664534** | 23/75 | United Kingdom (100%) | Europe (100%) | 2021-07-24 to 2021-08-23 | **Active** | AY.4 (93%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_I232T (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP4_V3F (100%) |
| **664556** | 49/226 | United Kingdom (100%) | Europe (100%) | 2021-07-24 to 2021-08-23 | **Active** | AY.6 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),Spike_A67V (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP2_D511N (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),NSP13_E365D (100%),Spike_T22I (100%),Spike_L452R (100%),NS7a_T120I (99%) |
| **664591** | 24/44 | United Kingdom (100%) | Europe (100%) | 2021-07-20 to 2021-08-21 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP3_S1230F (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP4_T492I (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),N_D63G (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_A488S (100%),Spike_G184S (100%),Spike_F157del (100%),NSP14_A394V (100%),NSP16_L111F (100%),N_D377Y (100%),NSP13_P77L (100%),NS3_P42L (100%),Spike_E156G (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_R765L (98%) |
| **664651** | 78/224 | United Kingdom (99%), Gibraltar (1%) | Europe (100%) | 2021-06-24 to 2021-08-21 | **Active** | AY.4 (59%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_N148S (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (100%),NS3_K235T (96%),NSP3_S1424F (70%) |
| **665026** | 20/25 | United Kingdom (88%), USA (8%), Portugal (4%) | Europe (92%) | 2021-07-19 to 2021-08-23 | **Active** | B.1.617.2 (80%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS3_T151I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (92%),Spike_G142D (92%),NSP3_A1711V (88%),NSP16_P236S (64%) |
| **666230** | 32/93 | United Kingdom (100%) | Europe (100%) | 2021-07-25 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_I1179V (99%),Spike_V16F (99%) |
| **667153** | 54/88 | United Kingdom (91%), Portugal (2%), France (1%), Belgium (1%), USA (1%) | Europe (99%) | 2021-06-09 to 2021-08-21 | **Active** | B.1.617.2 (91%) | GK (100%) | Delta | M_V23L (100%),N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_D950N (99%),NSP3_A1711V (95%),Spike_G142D (94%) |
| **667264** | 25/69 | United Kingdom (93%), Turkey (4%), Sweden (1%), USA (1%) | Europe (99%) | 2021-06-23 to 2021-08-21 | **Active** | AY.4 (84%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP4_V167L (96%),Spike_T95I (94%),Spike_G142D (94%),NSP3_A1711V (93%),NSP3_V732F (74%),NSP3_V965F (74%),NSP13_K584N (74%) |
| **668497** | 31/79 | United Kingdom (96%), USA (3%), Denmark (1%) | Europe (97%) | 2021-06-06 to 2021-08-22 | **Active** | AY.6 (97%) | GK (97%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),N_K248N (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP2_D511N (97%),NSP13_E365D (97%) |
| **669074** | 112/139 | Belgium (29%), Denmark (17%), France (14%), Netherlands (13%), Germany (10%) | Europe (96%) | 2021-06-11 to 2021-08-23 | **Active** | AY.4 (91%) | GK (99%) | Delta | N_G215C (100%),N_D63G (100%),NSP12_G671S (100%),Spike_T250I (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_Q613H (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),Spike_T299I (100%),Spike_T29A (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (99%),Spike_D950N (99%),NSP4_T492I (99%),Spike_G142D (89%) |
| **671583** | 48/103 | United Kingdom (97%), USA (2%), Germany (1%) | Europe (98%) | 2021-07-05 to 2021-08-22 | **Active** | B.1.617.2 (95%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS3_T223I (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (98%),NSP3_A1711V (97%),NSP2_G481S (83%),NSP2_A80T (81%),NSP3_E1252K (79%) |
| **671788** | 158/355 | United Kingdom (99%), Denmark (1%), Italy (0%) | Europe (100%) | 2021-06-17 to 2021-08-23 | **Active** | AY.4 (85%) | GK (99%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS3_A110T (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T19R (100%),NS7a_T120I (100%) |
| **671921** | 48/99 | United Kingdom (99%), USA (1%) | Europe (99%) | 2021-05-21 to 2021-08-22 | **Active** | AY.4 (90%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (94%),Spike_T95I (94%),Spike_T859I (93%) |
| **672076** | 37/95 | United Kingdom (100%) | Europe (100%) | 2021-07-26 to 2021-08-22 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_G18A (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),NSP3_S1230F (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP14_A274S (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **674013** | 64/86 | USA (91%), Mexico (9%) | North America (100%) | 2021-07-02 to 2021-08-21 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_T223I (100%),NS3_S26L (100%),NSP13_A296S (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),NSP12_P323L (99%),Spike_D950N (97%),NSP6_T77A (97%),Spike_G142D (88%) |
| **675049** | 33/78 | United Kingdom (100%) | Europe (100%) | 2021-07-15 to 2021-08-21 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | Spike_V1264L (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NSP6_T181I (100%),NS3_S26L (100%),NSP2_T528I (100%),NSP4_T143I (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),NS7a_L112I (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),Spike_D950N (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP9_T21I (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (100%),NSP5_Y239C (92%),NS7b_L34R (92%) |
| **675871** | 35/40 | United Kingdom (82%), USA (8%), France (2%), Germany (2%), Denmark (2%) | Europe (92%) | 2021-06-01 to 2021-08-20 | **Active** | B.1.617.2 (88%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (95%),Spike_G142D (95%),NSP3_A1711V (82%) |
| **676273** | 37/82 | United Kingdom (100%) | Europe (100%) | 2021-06-26 to 2021-08-23 | **Active** | AY.4 (100%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_R203M (100%),NSP12_G671S (100%),NSP3_E1091A (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),N_D63V (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_I397T (73%) |
| **677614** | 135/581 | Turkey (93%), Netherlands (2%), Germany (2%), Belgium (1%), USA (1%) | Europe (98%) | 2021-04-25 to 2021-08-18 | **Active** | B.1.617.2 (82%) | GK (100%) | Delta | N_R203M (100%),NS3_S26L (100%),NSP3_P1228L (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_D614G (100%),N_D63G (100%),Spike_I850L (100%),NSP12_G671S (99%),Spike_R158del (99%),Spike_F157del (99%),Spike_D950N (99%),Spike_E156G (99%),Spike_P681R (99%),NSP3_A488S (99%),NSP14_A394V (99%),NSP13_P77L (99%),Spike_L452R (99%),NSP4_T492I (99%),Spike_T95I (98%),NS3_K16T (98%),NS7a_V82A (96%),NSP12_P323L (96%),N_G215C (95%),NSP16_K160R (94%),NSP6_T77A (90%),Spike_G142D (81%),NSP4_V167L (69%) |
| **678470** | 58/73 | Belgium (19%), France (16%), Germany (16%), Switzerland (11%), Turkey (10%) | Europe (93%) | 2021-06-17 to 2021-08-17 | **Active** | B.1.617.2 (100%) | GK (99%) | Delta | NS7a_L116F (100%),N_R203M (100%),NSP12_G671S (100%),NSP2_P129L (100%),NS3_S26L (100%),NSP14_P46L (100%),Spike_P681R (100%),Spike_R158del (100%),NS7a_V82A (100%),Spike_F157del (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP6_V149A (100%),NSP3_P822L (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP4_A446V (100%),Spike_T478K (100%),NSP12_P323L (100%),NS3_Q57R (100%),Spike_D614G (100%),Spike_L452R (100%),NS3_E102V (99%),N_D63G (97%),Spike_D950N (92%) |
| **678499** | 133/341 | Turkey (72%), Germany (8%), Denmark (6%), USA (5%), Sweden (1%) | Europe (94%) | 2021-06-04 to 2021-08-23 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NS3_V202L (100%),NS7a_V82A (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_D614G (100%),Spike_R158del (100%),Spike_F157del (100%),Spike_E156G (100%),Spike_T478K (100%),NSP3_P1228L (100%),NSP3_A488S (100%),NSP4_T492I (100%),Spike_L452R (99%),Spike_D950N (98%),NSP12_P323L (98%),NSP14_A394V (98%),NSP3_P1469S (98%),Spike_T95I (97%),NSP6_T77A (97%),Spike_G142D (93%),Spike_P681R (92%),NSP4_V167L (81%) |
| **678759** | 45/77 | United Kingdom (100%) | Europe (100%) | 2021-05-31 to 2021-08-22 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_G215C (97%),NSP8_T124I (95%),NSP3_E1270G (95%) |
| **678833** | 29/75 | United Kingdom (100%) | Europe (100%) | 2021-07-16 to 2021-08-22 | **Active** | AY.4 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),N_S202I (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),Spike_A222V (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),Spike_Y145H (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T95I (99%),Spike_V36F (97%) |
| **679459** | 35/48 | United Kingdom (98%), USA (2%) | Europe (98%) | 2021-06-25 to 2021-08-22 | **Active** | B.1.617.2 (88%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (98%),NSP2_S263F (77%) |
| **680431** | 28/54 | USA (100%) | North America (100%) | 2021-06-22 to 2021-08-21 | **Active** | B.1.617.2 (96%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),NS8_C102F (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_V71I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP14_P140S (100%),NS7a_V93F (100%),NSP16_Q238H (98%),Spike_D950N (98%),Spike_G142D (96%),NSP12_D269G (78%) |
| **684071** | 24/39 | USA (100%) | North America (100%) | 2021-07-26 to 2021-08-20 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP13_I334V (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),Spike_G72E (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NS8_P36S (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),NSP3_S1877N (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%) |
| **687993** | 25/68 | United Kingdom (100%) | Europe (100%) | 2021-05-25 to 2021-08-22 | **Active** | AY.4 (90%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),N_K374N (99%),Spike_L452R (99%),NSP4_A231V (90%),NSP15_V172L (81%) |
| **689545** | 27/49 | United Kingdom (100%) | Europe (100%) | 2021-07-28 to 2021-08-22 | **Active** | B.1.617.2 (98%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NS7a_G38stop (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP3_G1447C (100%),NSP6_T77A (100%),NSP3_V418F (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP12_T76I (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_T22I (100%) |
| **690434** | 39/93 | United Kingdom (91%), Japan (5%), USA (2%), France (1%) | Europe (92%) | 2021-06-01 to 2021-08-17 | **Active** | B.1.617.2 (91%) | GK (99%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),Spike_Q173H (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS7a_V82A (96%),NSP3_A1711V (91%),NSP3_S1494L (85%),NSP12_A656V (85%) |
| **690939** | 34/56 | Turkey (55%), Germany (30%), Switzerland (5%), France (2%), United Kingdom (2%) | Europe (98%) | 2021-06-10 to 2021-08-19 | **Active** | AY.12 (55%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NS3_V202L (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),Spike_L452R (98%),N_D377Y (96%),NSP4_V167L (70%) |
| **692004** | 33/39 | USA (95%), Aruba (3%), United Kingdom (3%) | North America (95%) | 2021-07-08 to 2021-08-18 | **Active** | B.1.617.2 (92%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NSP16_Q238H (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NS7a_V71I (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NS3_L95S (97%),Spike_D950N (95%),Spike_G142D (92%) |
| **696243** | 27/31 | United Kingdom (97%), USA (3%) | Europe (97%) | 2021-06-02 to 2021-08-19 | **Active** | AY.4 (71%) | GK (97%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP3_A1711V (97%) |
| **700083** | 31/50 | United Kingdom (32%), Netherlands (24%), South Africa (8%), Sweden (8%), France (6%) | Europe (82%) | 2021-06-15 to 2021-08-23 | **Active** | B.1.617.2 (98%) | GK (98%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_M124I (92%) |
| **702163** | 33/39 | Cambodia (100%) | Asia (100%) | 2021-07-13 to 2021-08-23 | **Active** | B.1.1.7 (100%) | GR (97%) | Alpha | Spike_H69del (100%),NS8_Q27stop (100%),NSP3_T183I (100%),Spike_T716I (100%),NS8_K68stop (100%),NSP6_S106del (100%),Spike_E484K (100%),N_R203K (100%),Spike_A570D (100%),Spike_N501Y (100%),NSP3_I1412T (100%),NS8_R52I (100%),Spike_P681H (100%),Spike_Y144del (100%),NSP6_G107del (100%),NSP3_A890D (100%),Spike_D1118H (100%),NSP6_F108del (100%),NS8_Y73C (100%),N_G204R (100%),Spike_V70del (100%),NSP12_P323L (100%),Spike_D614G (100%),N_D3L (100%),Spike_S982A (100%),N_S235F (100%),NSP5_P241L (97%) |
| **703118** | 27/31 | Switzerland (52%), Germany (42%), Italy (6%) | Europe (100%) | 2021-07-27 to 2021-08-19 | **Active** | AY.12 (94%) | GK (100%) | Delta | N_G215C (100%),Spike_T95I (100%),N_D63G (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP16_D179G (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),N_R203M (97%),Spike_D950N (97%),NS7b_T40I (97%),NS8_I88T (97%),NSP13_H164Y (71%) |
| **707185** | 68/144 | United Kingdom (99%), Gibraltar (1%), France (1%) | Europe (100%) | 2021-05-11 to 2021-08-21 | **Active** | AY.4 (65%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),NS7a_G38V (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),Spike_G142D (99%),NSP3_A1711V (85%),Spike_T95I (85%),NSP5_T280N (67%),NSP7_M52V (63%) |
| **713879** | 41/125 | United Kingdom (100%) | Europe (100%) | 2021-06-29 to 2021-08-23 | **Active** | B.1.617.2 (100%) | GK (100%) | Delta | N_G215C (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),N_S327L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_G238C (100%),N_D377Y (100%),M_I82T (100%),NSP6_V91L (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_P918S (82%),NSP13_I370V (75%) |
| **715969** | 33/88 | United Kingdom (100%) | Europe (100%) | 2021-08-12 to 2021-08-22 | **Active** | B.1.617.2 (99%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_E554G (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (99%) |
| **719843** | 27/52 | United Kingdom (100%) | Europe (100%) | 2021-06-16 to 2021-08-22 | **Active** | AY.4 (87%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP12_H613Y (88%),NSP3_T1456I (88%) |
| **722884** | 54/151 | United Kingdom (100%) | Europe (100%) | 2021-06-18 to 2021-08-22 | **Active** | B.1.617.2 (97%) | GK (100%) | Delta | N_G215C (100%),NSP3_A1711V (100%),Spike_T95I (100%),N_D63G (100%),N_R203M (100%),NSP12_G671S (100%),Spike_G142D (100%),NS3_S26L (100%),NS7a_H73Y (100%),Spike_P681R (100%),Spike_R158del (100%),NSP3_P1228L (100%),NS7a_V82A (100%),NSP3_A488S (100%),Spike_F157del (100%),NSP4_T492I (100%),NSP14_A394V (100%),Spike_T19R (100%),NS7a_T120I (100%),N_D377Y (100%),M_I82T (100%),NSP12_D879N (100%),Spike_D950N (100%),NS7b_T40I (100%),NSP13_P77L (100%),Spike_E156G (100%),NSP3_P1469S (100%),Spike_T478K (100%),NSP6_T77A (100%),NSP12_P323L (100%),Spike_D614G (100%),NSP4_V167L (100%),Spike_L452R (100%),NSP5_K88R (99%),NSP3_P402L (90%) |
| **731055** | 4538/8651 | USA (39%), United Kingdom (35%), Turkey (15%), Denmark (2%), Canada (1%) | Europe (57%) | 2021-01-07 to 2021-08-24 | **Active** | B.1.617.2 (72%) | GK (99%) | Delta | N_D377Y (100%),M_I82T (100%),Spike_D614G (100%),N_D63G (100%),N_R203M (100%),NSP13_P77L (100%),NSP12_P323L (100%),Spike_T19R (100%),Spike_T478K (100%),Spike_P681R (100%),Spike_R158del (100%),Spike_E156G (100%),NS3_S26L (100%),Spike_F157del (100%),NS7a_V82A (100%),NS7a_T120I (100%),Spike_D950N (100%),N_G215C (99%),Spike_L452R (99%),NSP12_G671S (99%),NSP3_P1469S (99%),NSP3_P1228L (99%),NSP14_A394V (99%),NSP4_V167L (97%),NSP4_T492I (96%),Spike_G142D (95%),NSP6_T77A (94%),NS7b_T40I (93%),NSP3_A488S (93%) |
